@@ -6,7 +6,7 @@
 import xml.etree.ElementTree as ElementTree
 import json
 from collections import defaultdict
-from typing import Any
+from typing import Any, Dict
 from leo.core import leoGlobals as g
 from leo.core import leoNodes
 from leo.core import leoFileCommands
@@ -21,7 +21,10 @@ def copyOutline(self, event=None):
     c.endEditing()
     s = c.fileCommands.outline_to_clipboard_string()
     g.app.paste_c = c
+    if g.app.inBridge:
+        return s
     g.app.gui.replaceClipboardWith(s)
+    return s
 #@+node:ekr.20220314071523.1: *3* c_oc.copyOutlineAsJson & helpers
 @g.commander_command('copy-node-as-json')
 def copyOutlineAsJSON(self, event=None):
@@ -54,15 +57,21 @@ def copyOutlineAsJSON(self, event=None):
     def outline_to_json(c):
         """Return the JSON representation of c."""
         positions = list(c.p.self_and_subtree())
+        uas_dict: Dict[str, Any] = {}
+        for p in positions:
+            if p.u:
+                try:
+                    uas_dict[p.v.gnx] = json.dumps(p.u, skipkeys=True)
+                except TypeError:
+                    g.trace(f"Can not serialize uA for {p.h}", g.callers(6))
+                    # g.printObj(p.u)
         d = {
             'leoHeader': {'fileFormat': 2},
             'globals': json_globals(c),
             'tnodes': {
                 p.v.gnx: p.v._bodyString for p in positions
             },
-            'uas': {
-                p.v.gnx: json.dumps(p.u, skipkeys=True) for p in positions if p.u
-            },
+            'uas': uas_dict,
             'vnodes': [
                 json_vnode(c.p.v)
             ],
@@ -73,6 +82,8 @@ def copyOutlineAsJSON(self, event=None):
     c.endEditing()
     s = outline_to_json(c)
     g.app.paste_c = c
+    if g.app.inBridge:
+        return s
     g.app.gui.replaceClipboardWith(s)
 #@+node:ekr.20031218072017.1549: *3* c_oc.cutOutline
 @g.commander_command('cut-node')
@@ -209,9 +220,7 @@ def pasteAsTemplate(self, event=None):
     if not s or not c.canPasteOutline(s):
         return None  # This should never happen.
 
-    isJson = False
-    if s.lstrip().startswith("{"):
-        isJson = True
+    isJson = s.lstrip().startswith("{")
 
     #@+others
     #@+node:vitalije.20200529112224.1: *4* skip_root
@@ -337,6 +346,7 @@ def pasteAsTemplate(self, event=None):
 
     xvelements: Any
     xtelements: Any
+    uas: Any  # Possible bug?
 
     if not isJson:
         xroot = ElementTree.fromstring(g.app.gui.getTextFromClipboard())
@@ -638,7 +648,7 @@ def expandNodeOrGoToFirstChild(self, event=None):
 #@+node:ekr.20060928062431: *3* c_oc.expandOnlyAncestorsOfNode
 @g.commander_command('expand-ancestors-only')
 def expandOnlyAncestorsOfNode(self, event=None, p=None):
-    """Contract all nodes in the outline."""
+    """Contract all nodes except ancestors of the selected node."""
     c = self
     level = 1
     if p:
@@ -1136,7 +1146,7 @@ def insertHeadline(self, event=None, op_name="Insert Node", as_child=False):
     """
     c = self
     # Fix #600.
-    return insertHeadlineHelper(c, event=event, as_child=as_child)
+    return insertHeadlineHelper(c, event=event, as_child=as_child, op_name=op_name)
 
 @g.commander_command('insert-as-first-child')
 def insertNodeAsFirstChild(self, event=None):
@@ -1769,6 +1779,7 @@ def cantMoveMessage(c):
 #@+node:ekr.20180201040936.1: ** count-children
 @g.command('count-children')
 def count_children(event=None):
+    """Print out the number of children for the currently selected node"""
     c = event and event.get('c')
     if c:
         g.es_print(f"{c.p.numberOfChildren()} children")
