@@ -3,8 +3,8 @@
 #@+node:ekr.20150514041209.1: * @file ../commands/editFileCommands.py
 #@@first
 """Leo's file-editing commands."""
-#@+<< imports >>
-#@+node:ekr.20170806094317.4: ** << imports >> (editFileCommands.py)
+#@+<< editFileCommands imports >>
+#@+node:ekr.20170806094317.4: ** << editFileCommands imports >>
 import difflib
 import io
 import os
@@ -13,7 +13,7 @@ from typing import Any, List
 from leo.core import leoGlobals as g
 from leo.core import leoCommands
 from leo.commands.baseCommands import BaseEditCommandsClass
-#@-<< imports >>
+#@-<< editFileCommands imports >>
 
 def cmd(name):
     """Command decorator for the EditFileCommandsClass class."""
@@ -209,8 +209,10 @@ class EditFileCommandsClass(BaseEditCommandsClass):
                         total += 1
                         c.undoer.afterChangeNodeContents(p2, undoType, bunch2)
                 g.es_print(f"{n} node{g.plural(n)} {p.h}")
-        if total > 0:
-            c.undoer.afterChangeGroup(c.p, undoType)
+        # Call this only once, at end.
+        c.undoer.afterChangeGroup(c.p, undoType)
+        if total == 0:
+            g.es("Command did not find any whitespace to adjust")
         g.es_print(f"{total} total node{g.plural(total)}")
     #@+node:ekr.20170806094319.8: *4* efc.cleanAtCleanNode
     def cleanAtCleanNode(self, p, undoType):
@@ -741,6 +743,7 @@ class GitDiffController:
         if aList:
             devel_rev = aList[0]
             devel_rev = devel_rev[:8]
+            g.trace('devel_rev', devel_rev)
             self.diff_two_revs(
                 rev1=devel_rev,  # Before: Latest devel commit.
                 rev2='HEAD',  # After: Latest branch commit
@@ -782,7 +785,8 @@ class GitDiffController:
         Create an outline describing the git diffs for all files changed
         between rev1 and rev2.
         """
-        c = self.c
+        c, u = self.c, self.c.undoer
+
         if not self.get_directory():
             return
         # Get list of changed files.
@@ -792,13 +796,24 @@ class GitDiffController:
         if n > 5:
             message += ". This may take awhile..."
         g.es_print(message)
+
+        undoType = 'Git Diff'
+        c.selectPosition(c.lastTopLevel())  # pre-select to help undo-insert
+        u.beforeChangeGroup(c.p, undoType)
         # Create the root node.
+        undoData = u.beforeInsertNode(c.p)  # c.p is subject of 'insertAfter'
         self.root = c.lastTopLevel().insertAfter()
         self.root.h = f"git diff revs: {rev1} {rev2}"
         self.root.b = '@ignore\n@nosearch\n'
+        u.afterInsertNode(self.root, 'Create diff root node', undoData)
+
         # Create diffs of all files.
         for fn in files:
+            undoData = u.beforeChangeTree(self.root)
             self.diff_file(fn=fn, rev1=rev1, rev2=rev2)
+            u.afterChangeTree(self.root, undoType, undoData)
+
+        u.afterChangeGroup(c.p, undoType=undoType)
         self.finish()
     #@+node:ekr.20170806094320.12: *4* gdc.git_diff & helper
     def git_diff(self, rev1='HEAD', rev2=''):
@@ -825,9 +840,19 @@ class GitDiffController:
         """Diff all files given by rev1 and rev2."""
         files = self.get_files(rev1, rev2)
         if files:
-            self.root = self.create_root(rev1, rev2)
+            c, u = self.c, self.c.undoer
+            undoType = 'Git Diff'
+            c.selectPosition(c.lastTopLevel())  # pre-select to help undo-insert
+            u.beforeChangeGroup(c.p, undoType)
+
+            self.root = self.create_root(rev1, rev2)  # creates it's own undo bead
+
             for fn in files:
+                undoData = u.beforeChangeTree(self.root)
                 self.diff_file(fn=fn, rev1=rev1, rev2=rev2)
+                u.afterChangeTree(self.root, undoType, undoData)
+
+            u.afterChangeGroup(c.p, undoType=undoType)
             self.finish()
         return bool(files)
     #@+node:ekr.20180510095801.1: *3* gdc.Utils
@@ -889,13 +914,17 @@ class GitDiffController:
     def create_file_node(self, diff_list, fn):
         """Create an organizer node for the file."""
         p = self.root.insertAsLastChild()
-        p.h = fn.strip()
+        p.h = 'diff: ' + fn.strip()
         p.b = ''.join(diff_list)
         return p
     #@+node:ekr.20170806094320.18: *4* gdc.create_root
     def create_root(self, rev1, rev2):
         """Create the top-level organizer node describing the git diff."""
-        c = self.c
+        c, u = self.c, self.c.undoer
+        undoType = 'Create diff root node'  # Same undoType is reused for all inner undos
+        c.selectPosition(c.lastTopLevel())  # pre-select to help undo-insert
+        undoData = u.beforeInsertNode(c.p)  # c.p is subject of 'insertAfter'
+
         r1, r2 = rev1 or '', rev2 or ''
         p = c.lastTopLevel().insertAfter()
         p.h = f"git diff {r1} {r2}"
@@ -906,6 +935,8 @@ class GitDiffController:
                 f"{r2}={self.get_revno(r2)}")
         else:
             p.b += f"{r1}={self.get_revno(r1)}"
+
+        u.afterInsertNode(p, undoType, undoData)
         return p
     #@+node:ekr.20170806094320.7: *4* gdc.find_file
     def find_file(self, fn):
