@@ -1,7 +1,9 @@
+# -*- coding: utf-8 -*-
 #@+leo-ver=5-thin
 #@+node:ekr.20031218072017.2794: * @file leoColor.py
-#@+<< docstring >>
-#@+node:bob.20080115083029: ** << docstring >>
+#@@first
+#@+<< leoColor docstring >>
+#@+node:bob.20080115083029: ** << leoColor docstring >>
 """A color database for Leo.
 
 leo_color_database is a dictionary of color names mapped onto the
@@ -24,24 +26,30 @@ These are::
 
 Use these functions as follows::
 
-    import leo.core.leoColor as leoColor
+    from leo.core import leoColor
     leoColor.getRGB(name, default)
 
 If neither 'name' nor 'default' can be translated then accessor functions
 will return None.
 """
-#@-<< docstring >>
-import leo.core.leoGlobals as g
-# import re
-# import string
+#@-<< leoColor docstring >>
 #@+<< define leo_color_database >>
 #@+node:bob.20080115070511.2: ** << define leo_color_database >>
-# All keys should contain only lower-case letters.
+#@@language rest
+#@+at
+# All names added to this database should be in normalized form,
+# otherwise the accessor functions won't work.
+#
+# Adding names here will make them availiable to all gui's and
+# dhtml that use this service.
+#
+# Names are normalized by removing spaces and capitalization.
+#@@c
 leo_color_database = {
     # leo colors
-    "leoblue": "#F0F8FF", #alice blue
+    "leoblue": "#F0F8FF",  #alice blue
     "leoyellow": "#ffffec",
-    "leopink": "#FFE4E1", # misty rose
+    "leopink": "#FFE4E1",  # misty rose
     # Solarized colors
     'altsolarizedgreen': '#84c805',
     'darkbase0': '#fdffdd',
@@ -55,7 +63,7 @@ leo_color_database = {
     'darkbase1': '#93a1a1',
     'darkbase2': '#eee8d5',
     'darkbase3': '#fdf6e3',
-    'solarizedblack': '#000005', # Unofficial.
+    'solarizedblack': '#000005',  # Unofficial.
     'solarizedblue': '#268bd2',
     'solarizedcyan': '#2aa198',
     'solarizedgreen': '#859900',
@@ -724,12 +732,12 @@ leo_color_database = {
     "yellowgreen": "#9ACD32"
 }
 #@-<< define leo_color_database >>
-#
+
 # Check that all keys are normalized.
 # This is essential for Leo's jEdit-based colorizers.
 for key in leo_color_database:
     if not key.isalnum():
-        print('\nleoColor.py: non-normalized key: %s\n' % repr(key))
+        print(f'\nleoColor.py: non-normalized key: {key!r}\n')
 #@+others
 #@+node:bob.20080115070511.3: ** color database functions
 #@+node:bob.20071231111744.2: *3* get / getColor
@@ -743,7 +751,7 @@ def getColor(name, default=None):
     If 'name' is not in global_color_database then getColor(default, None)
     is called and that result returned.
     """
-    if not g.isString(name):
+    if not isinstance(name, str):
         return name
     if name[0] == '#':
         return name
@@ -761,7 +769,7 @@ def getColorRGB(name, default=None):
     """Convert a named color into an (r, g, b) tuple."""
     s = getColor(name, default)
     try:
-        color = int(s[1: 3], 16), int(s[3: 5], 16), int(s[5: 7], 16)
+        color = int(s[1:3], 16), int(s[3:5], 16), int(s[5:7], 16)
     except Exception:
         color = None
     return color
@@ -773,10 +781,9 @@ def getColorCairo(name, default=None):
     # pylint: disable=unpacking-non-sequence
     color = getColorRGB(name, default)
     if color is None:
-        return
-    else:
-        r, g, b = color
-        return r / 255.0, g / 255.0, b / 255.0
+        return None
+    r, g, b = color
+    return r / 255.0, g / 255.0, b / 255.0
 
 getCairo = getColorCairo
 #@-others
