@@ -1,5 +1,7 @@
+# -*- coding: utf-8 -*-
 #@+leo-ver=5-thin
 #@+node:ekr.20180212072657.2: * @file leoCompare.py
+#@@first
 """Leo's base compare class."""
 import difflib
 import filecmp
@@ -462,13 +464,22 @@ class CompareLeoOutlines:
         if len(aList) < 2:
             g.trace('Not enough files in', repr(aList))
             return
-        self.root = self.create_root(aList)
+        c, u = self.c, self.c.undoer
+        undoType = 'Diff Leo files'
+        u.beforeChangeGroup(c.p, undoType)
+
+        self.root = self.create_root(aList)  # creates it's own undo bead
+
         self.visible = visible
         while len(aList) > 1:
             path1 = aList[0]
             aList = aList[1:]
             for path2 in aList:
-                self.diff_two_files(path1, path2)
+                undoData = u.beforeChangeTree(self.root)
+                self.diff_two_files(path1, path2)  # adds to self.root
+                u.afterChangeTree(self.root, undoType, undoData)
+
+        u.afterChangeGroup(c.p, undoType=undoType)
         self.finish()
     #@+node:ekr.20180211170333.3: *3* loc.diff_two_files
     def diff_two_files(self, fn1, fn2):
@@ -556,10 +567,16 @@ class CompareLeoOutlines:
     #@+node:ekr.20180211170333.8: *4* loc.create_root
     def create_root(self, aList):
         """Create the top-level organizer node describing all the diffs."""
-        c = self.c
+        c, u = self.c, self.c.undoer
+        undoType = 'Create diff root node'  # Same undoType is reused for all inner undos
+        c.selectPosition(c.lastTopLevel())  # pre-select to help undo-insert
+        undoData = u.beforeInsertNode(c.p)  # c.p is subject of 'insertAfter'
+
         p = c.lastTopLevel().insertAfter()
         p.h = 'diff-leo-files'
         p.b = '\n'.join(aList) + '\n'
+
+        u.afterInsertNode(p, undoType, undoData)
         return p
     #@+node:ekr.20180211170333.10: *4* loc.finish
     def finish(self):
@@ -631,25 +648,34 @@ def diff_leo_files(event):
 @g.command('diff-marked-nodes')
 def diffMarkedNodes(event):
     """
-    When two or more nodes are marked, this command does the following:
+    When two or more nodes are marked, this command creates a
+    "diff marked node" as the last top-level node. The body of
+    this node contains "diff n" nodes, one for each pair of compared
+    nodes.
 
-    - Creates a "diff marked node" as the last top-level node. The body of
-      this node contains "diff n" nodes, one for each pair of compared
-      nodes.
+    Each diff n contains the diffs between the two diffed nodes, that is,
+    difflib.Differ().compare(p1.b, p2.b).
 
-    - Each diff n contains the diffs between the two diffed nodes, that is,
-      difflib.Differ().compare(p1.b, p2.b).  The children of the diff n are
-      *clones* of the two compared nodes.
+    The children of the diff n are clones of the two compared nodes.
     """
     c = event and event.get('c')
     if not c:
         return
+    u = c.undoer
+    undoType = 'Diff marked nodes'  # Same undoType is reused for all inner undos
+
     aList = [z for z in c.all_unique_positions() if z.isMarked()]
     n = 0
     if len(aList) >= 2:
+        u.beforeChangeGroup(c.p, undoType)  # going to perform many operations
+
+        c.selectPosition(c.lastTopLevel())  # pre-select to help undo-insert
+        undoData = u.beforeInsertNode(c.p)  # c.p is subject of 'insertAfter'
         root = c.lastTopLevel().insertAfter()
         root.h = 'diff marked nodes'
         root.b = '\n'.join([z.h for z in aList]) + '\n'
+        u.afterInsertNode(root, 'Create diff root node', undoData)
+
         while len(aList) > 1:
             n += 1
             p1, p2 = aList[0], aList[1]
@@ -657,16 +683,25 @@ def diffMarkedNodes(event):
             lines = difflib.Differ().compare(
                 g.splitLines(p1.b.rstrip() + '\n'),
                 g.splitLines(p2.b.rstrip() + '\n'))
+
+            undoData = u.beforeInsertNode(c.p)  # c.p is subject of 'insertAfter'
             p = root.insertAsLastChild()
             # p.h = 'Compare: %s, %s' % (g.truncate(p1.h, 22), g.truncate(p2.h, 22))
             p.h = f"diff {n}"
             p.b = f"1: {p1.h}\n2: {p2.h}\n{''.join(list(lines))}"
+            u.afterInsertNode(p, undoType, undoData)
+
+            undoData = u.beforeChangeTree(p)
             for p3 in (p1, p2):
                 clone = p3.clone()
                 clone.moveToLastChildOf(p)
+            u.afterChangeTree(p, undoType, undoData)
+
         root.expand()
         # c.unmarkAll()
         c.selectPosition(root)
+        # Add the changes to the outer undo group.
+        u.afterChangeGroup(c.p, undoType=undoType)
         c.redraw()
     else:
         g.es_print('Please mark at least 2 nodes')
