@@ -1,5 +1,8 @@
+# -*- coding: utf-8 -*-
 #@+leo-ver=5-thin
 #@+node:ekr.20031218072017.3603: * @file leoUndo.py
+#@@first
+
 # Suppress all mypy errors (mypy doesn't like g.Bunch).
 # type: ignore
 """Leo's undo/redo manager."""
@@ -42,11 +45,25 @@
 #
 # I first saw this model of unlimited undo in the documentation for Apple's Yellow Box classes.
 #@-<< How Leo implements unlimited undo >>
+#@+<< leoUndo imports >>
+#@+node:ekr.20220821074023.1: ** << leoUndo imports >>
+from typing import Any, Callable, TYPE_CHECKING
 from leo.core import leoGlobals as g
+#@-<< leoUndo imports >>
+#@+<< leoUndo annotations >>
+#@+node:ekr.20220821074054.1: ** << leoUndo annotations >>
+if TYPE_CHECKING:
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoNodes import Position, VNode
+else:
+    Cmdr = Position = VNode = Any
+Event = Any
+Wrapper = Any
+#@-<< leoUndo annotations >>
 # pylint: disable=unpacking-non-sequence
 #@+others
 #@+node:ekr.20150509193222.1: ** u.cmd (decorator)
-def cmd(name):
+def cmd(name: Any) -> Callable:
     """Command decorator for the Undoer class."""
     return g.new_cmd_decorator(name, ['c', 'undoer',])
 #@+node:ekr.20031218072017.3605: ** class Undoer
@@ -58,7 +75,7 @@ class Undoer:
     #@+others
     #@+node:ekr.20150509193307.1: *3* u.Birth
     #@+node:ekr.20031218072017.3606: *4* u.__init__
-    def __init__(self, c):
+    def __init__(self, c: Cmdr) -> None:
         self.c = c
         self.granularity = None  # Set in reloadSettings.
         self.max_undo_stack_size = c.config.getInt('max-undo-stack-size') or 0
@@ -117,7 +134,7 @@ class Undoer:
         self.verboseUndoGroup = None
         self.reloadSettings()
     #@+node:ekr.20191213085126.1: *4* u.reloadSettings
-    def reloadSettings(self):
+    def reloadSettings(self) -> None:
         """Undoer.reloadSettings."""
         c = self.c
         self.granularity = c.config.getString('undo-granularity')
@@ -127,13 +144,13 @@ class Undoer:
             self.granularity = 'line'
     #@+node:ekr.20050416092908.1: *3* u.Internal helpers
     #@+node:ekr.20031218072017.3607: *4* u.clearOptionalIvars
-    def clearOptionalIvars(self):
+    def clearOptionalIvars(self) -> None:
         u = self
         u.p = None  # The position/node being operated upon for undo and redo.
         for ivar in u.optionalIvars:
             setattr(u, ivar, None)
     #@+node:ekr.20060127052111.1: *4* u.cutStack
-    def cutStack(self):
+    def cutStack(self) -> None:
         u = self
         n = u.max_undo_stack_size
         if u.bead >= n > 0 and not g.unitTesting:
@@ -151,7 +168,7 @@ class Undoer:
         if 'undo' in g.app.debug and 'verbose' in g.app.debug:  # pragma: no cover
             print(f"u.cutStack: {len(u.beads):3}")
     #@+node:ekr.20080623083646.10: *4* u.dumpBead
-    def dumpBead(self, n):  # pragma: no cover
+    def dumpBead(self, n: int) -> str:  # pragma: no cover
         u = self
         if n < 0 or n >= len(u.beads):
             return 'no bead: n = ', n
@@ -163,14 +180,14 @@ class Undoer:
             result.append(f"{ivar} = {getattr(self, ivar)}")
         return '\n'.join(result)
 
-    def dumpTopBead(self):  # pragma: no cover
+    def dumpTopBead(self) -> str:  # pragma: no cover
         u = self
         n = len(u.beads)
         if n > 0:
             return self.dumpBead(n - 1)
         return '<no top bead>'
     #@+node:EKR.20040526150818: *4* u.getBead
-    def getBead(self, n):
+    def getBead(self, n: int) -> Any:
         """Set Undoer ivars from the bunch at the top of the undo stack."""
         u = self
         if n < 0 or n >= len(u.beads):
@@ -181,14 +198,14 @@ class Undoer:
             print(f" u.getBead: {n:3} of {len(u.beads)}")
         return bunch
     #@+node:EKR.20040526150818.1: *4* u.peekBead
-    def peekBead(self, n):
+    def peekBead(self, n: int) -> Any:
 
         u = self
         if n < 0 or n >= len(u.beads):
             return None
         return u.beads[n]
     #@+node:ekr.20060127113243: *4* u.pushBead
-    def pushBead(self, bunch):
+    def pushBead(self, bunch: Any) -> None:
         u = self
         # New in 4.4b2:  Add this to the group if it is being accumulated.
         bunch2 = u.bead >= 0 and u.bead < len(u.beads) and u.beads[u.bead]
@@ -204,17 +221,17 @@ class Undoer:
         if 'undo' in g.app.debug:  # pragma: no cover
             print(f"u.pushBead: {len(u.beads):3} {bunch.undoType}")
     #@+node:ekr.20031218072017.3613: *4* u.redoMenuName, undoMenuName
-    def redoMenuName(self, name):
+    def redoMenuName(self, name: str) -> str:
         if name == "Can't Redo":
             return name
         return "Redo " + name
 
-    def undoMenuName(self, name):
+    def undoMenuName(self, name: str) -> str:
         if name == "Can't Undo":
             return name
         return "Undo " + name
     #@+node:ekr.20060127070008: *4* u.setIvarsFromBunch
-    def setIvarsFromBunch(self, bunch):
+    def setIvarsFromBunch(self, bunch: Any) -> None:
         u = self
         u.clearOptionalIvars()
         if False and not g.unitTesting:  # Debugging. # pragma: no cover
@@ -234,7 +251,7 @@ class Undoer:
     #@+node:ekr.20031218072017.3614: *4* u.setRedoType
     # These routines update both the ivar and the menu label.
 
-    def setRedoType(self, theType):
+    def setRedoType(self, theType: str) -> None:
 
         u = self
         frame = u.c.frame
@@ -257,7 +274,7 @@ class Undoer:
             u.redoMenuLabel = name
             u.realRedoMenuLabel = realLabel
     #@+node:ekr.20091221145433.6381: *4* u.setUndoType
-    def setUndoType(self, theType):
+    def setUndoType(self, theType: str) -> None:
 
         u = self
         frame = u.c.frame
@@ -281,7 +298,7 @@ class Undoer:
             u.undoMenuLabel = name
             u.realUndoMenuLabel = realLabel
     #@+node:ekr.20031218072017.3616: *4* u.setUndoTypes
-    def setUndoTypes(self):
+    def setUndoTypes(self) -> None:
 
         u = self
         # Set the undo type and undo menu label.
@@ -298,7 +315,7 @@ class Undoer:
             u.setRedoType("Can't Redo")
         u.cutStack()
     #@+node:EKR.20040530121329: *4* u.restoreTree & helpers
-    def restoreTree(self, treeInfo):
+    def restoreTree(self, treeInfo: Any) -> None:
         """Use the tree info to restore all VNode data,
         including all links."""
         u = self
@@ -306,7 +323,7 @@ class Undoer:
         for v, vInfo in treeInfo:
             u.restoreVnodeUndoInfo(vInfo)
     #@+node:ekr.20050415170737.2: *5* u.restoreVnodeUndoInfo
-    def restoreVnodeUndoInfo(self, bunch):
+    def restoreVnodeUndoInfo(self, bunch: Any) -> None:
         """Restore all ivars saved in the bunch."""
         v = bunch.v
         v.statusBits = bunch.statusBits
@@ -317,7 +334,7 @@ class Undoer:
             v.unknownAttributes = uA
             v._p_changed = True
     #@+node:ekr.20050415170812.2: *5* u.restoreTnodeUndoInfo
-    def restoreTnodeUndoInfo(self, bunch):
+    def restoreTnodeUndoInfo(self, bunch: Any) -> None:
         v = bunch.v
         v.h = bunch.headString
         v.b = bunch.bodyString
@@ -327,7 +344,7 @@ class Undoer:
             v.unknownAttributes = uA
             v._p_changed = True
     #@+node:EKR.20040528075307: *4* u.saveTree & helpers
-    def saveTree(self, p, treeInfo=None):
+    def saveTree(self, p: Position, treeInfo: Any=None) -> Any:
         """Return a list of tuples with all info needed to handle a general undo operation."""
         # WARNING: read this before doing anything "clever"
         #@+<< about u.saveTree >>
@@ -363,7 +380,7 @@ class Undoer:
             child = child.next()
         return treeInfo
     #@+node:ekr.20050415170737.1: *5* u.createVnodeUndoInfo
-    def createVnodeUndoInfo(self, v):
+    def createVnodeUndoInfo(self, v: VNode) -> Any:
         """Create a bunch containing all info needed to recreate a VNode for undo."""
         bunch = g.Bunch(
             v=v,
@@ -375,12 +392,12 @@ class Undoer:
             bunch.unknownAttributes = v.unknownAttributes
         return bunch
     #@+node:ekr.20050525151449: *4* u.trace
-    def trace(self):  # pragma: no cover
+    def trace(self) -> None:  # pragma: no cover
         ivars = ('kind', 'undoType')
         for ivar in ivars:
             g.pr(ivar, getattr(self, ivar))
     #@+node:ekr.20050410095424: *4* u.updateMarks
-    def updateMarks(self, oldOrNew):
+    def updateMarks(self, oldOrNew: str) -> None:
         """Update dirty and marked bits."""
         u = self
         c = u.c
@@ -400,7 +417,7 @@ class Undoer:
     #@+node:ekr.20031218072017.3608: *3* u.Externally visible entries
     #@+node:ekr.20050318085432.4: *4* u.afterX...
     #@+node:ekr.20201109075104.1: *5* u.afterChangeBody
-    def afterChangeBody(self, p, command, bunch):
+    def afterChangeBody(self, p: Position, command: str, bunch: Any) -> None:
         """
         Create an undo node using d created by beforeChangeNode.
 
@@ -438,7 +455,7 @@ class Undoer:
                 'Call u.doTyping instead')
         u.updateAfterTyping(p, w)
     #@+node:ekr.20050315134017.4: *5* u.afterChangeGroup
-    def afterChangeGroup(self, p, undoType, reportFlag=False):
+    def afterChangeGroup(self, p: Position, undoType: str, reportFlag: bool=False) -> None:
         """
         Create an undo node for general tree operations using d created by
         beforeChangeGroup
@@ -477,7 +494,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20050315134017.2: *5* u.afterChangeNodeContents
-    def afterChangeNodeContents(self, p, command, bunch):
+    def afterChangeNodeContents(self, p: Position, command: str, bunch: Any) -> None:
         """Create an undo node using d created by beforeChangeNode."""
         u = self
         c = self.c
@@ -501,7 +518,7 @@ class Undoer:
         bunch.newYScroll = w.getYScrollPosition() if w else 0
         u.pushBead(bunch)
     #@+node:ekr.20201107145642.1: *5* u.afterChangeHeadline
-    def afterChangeHeadline(self, p, command, bunch):
+    def afterChangeHeadline(self, p: Position, command: str, bunch: Any) -> None:
         """Create an undo node using d created by beforeChangeHeadline."""
         u = self
         if u.redoing or u.undoing:
@@ -516,7 +533,7 @@ class Undoer:
 
     afterChangeHead = afterChangeHeadline
     #@+node:ekr.20050315134017.3: *5* u.afterChangeTree
-    def afterChangeTree(self, p, command, bunch):
+    def afterChangeTree(self, p: Position, command: str, bunch: Any) -> None:
         """Create an undo node for general tree operations using d created by beforeChangeTree"""
         u = self
         c = self.c
@@ -534,7 +551,7 @@ class Undoer:
         bunch.newTree = u.saveTree(p)
         u.pushBead(bunch)
     #@+node:ekr.20050424161505: *5* u.afterClearRecentFiles
-    def afterClearRecentFiles(self, bunch):
+    def afterClearRecentFiles(self, bunch: Any) -> None:
         u = self
         bunch.newRecentFiles = g.app.config.recentFiles[:]
         bunch.undoType = 'Clear Recent Files'
@@ -543,7 +560,7 @@ class Undoer:
         u.pushBead(bunch)
         return bunch
     #@+node:ekr.20111006060936.15639: *5* u.afterCloneMarkedNodes
-    def afterCloneMarkedNodes(self, p):
+    def afterCloneMarkedNodes(self, p: Position) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -563,7 +580,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20160502175451.1: *5* u.afterCopyMarkedNodes
-    def afterCopyMarkedNodes(self, p):
+    def afterCopyMarkedNodes(self, p: Position) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -583,7 +600,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050411193627.5: *5* u.afterCloneNode
-    def afterCloneNode(self, p, command, bunch):
+    def afterCloneNode(self, p: Position, command: str, bunch: Any) -> None:
         u = self
         if u.redoing or u.undoing:
             return  # pragma: no cover
@@ -599,7 +616,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050411193627.8: *5* u.afterDeleteNode
-    def afterDeleteNode(self, p, command, bunch):
+    def afterDeleteNode(self, p: Position, command: str, bunch: Any) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -613,7 +630,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20111005152227.15555: *5* u.afterDeleteMarkedNodes
-    def afterDeleteMarkedNodes(self, data, p):
+    def afterDeleteMarkedNodes(self, data: Any, p: Position) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -629,7 +646,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20080425060424.8: *5* u.afterDemote
-    def afterDemote(self, p, followingSibs):
+    def afterDemote(self, p: Position, followingSibs: Any) -> None:
         """Create an undo node for demote operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -645,7 +662,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20050411193627.9: *5* u.afterInsertNode
-    def afterInsertNode(self, p, command, bunch):
+    def afterInsertNode(self, p: Position, command: str, bunch: Any) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -668,7 +685,7 @@ class Undoer:
             bunch.afterTree = afterTree
         u.pushBead(bunch)
     #@+node:ekr.20050526124257: *5* u.afterMark
-    def afterMark(self, p, command, bunch):
+    def afterMark(self, p: Position, command: str, bunch: Any) -> None:
         """Create an undo node for mark and unmark commands."""
         # 'command' unused, but present for compatibility with similar methods.
         u = self
@@ -680,7 +697,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050410110343: *5* u.afterMoveNode
-    def afterMoveNode(self, p, command, bunch):
+    def afterMoveNode(self, p: Position, command: str, bunch: Any) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -697,7 +714,7 @@ class Undoer:
         bunch.newP = p.copy()
         u.pushBead(bunch)
     #@+node:ekr.20080425060424.12: *5* u.afterPromote
-    def afterPromote(self, p, children):
+    def afterPromote(self, p: Position, children: Any) -> None:
         """Create an undo node for demote operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -713,7 +730,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20080425060424.2: *5* u.afterSort
-    def afterSort(self, p, bunch):
+    def afterSort(self, p: Position, bunch: Any) -> None:
         """Create an undo node for sort operations"""
         u = self
         # c = self.c
@@ -723,7 +740,7 @@ class Undoer:
         u.setUndoTypes()
     #@+node:ekr.20050318085432.3: *4* u.beforeX...
     #@+node:ekr.20201109074740.1: *5* u.beforeChangeBody
-    def beforeChangeBody(self, p):
+    def beforeChangeBody(self, p: Position) -> None:
         """Return data that gets passed to afterChangeBody."""
         w = self.c.frame.body.wrapper
         bunch = self.createCommonBunch(p)  # Sets u.oldMarked, u.oldSel, u.p
@@ -735,7 +752,7 @@ class Undoer:
     #@+node:ekr.20050315134017.7: *5* u.beforeChangeGroup
     changeGroupWarning = False
 
-    def beforeChangeGroup(self, p, command, verboseUndoGroup=True):
+    def beforeChangeGroup(self, p: Position, command: str, verboseUndoGroup: bool=False) -> None:
         """Prepare to undo a group of undoable operations."""
         c, u = self.c, self
         if p != c.p:  # Prepare to ignore p argument.
@@ -756,7 +773,7 @@ class Undoer:
         u.bead += 1
         u.beads[u.bead:] = [bunch]
     #@+node:ekr.20201107145859.1: *5* u.beforeChangeHeadline
-    def beforeChangeHeadline(self, p):
+    def beforeChangeHeadline(self, p: Position) -> None:
         """
         Return data that gets passed to afterChangeNode.
 
@@ -769,7 +786,7 @@ class Undoer:
 
     beforeChangeHead = beforeChangeHeadline
     #@+node:ekr.20050315133212.2: *5* u.beforeChangeNodeContents
-    def beforeChangeNodeContents(self, p):
+    def beforeChangeNodeContents(self, p: Position) -> None:
         """Return data that gets passed to afterChangeNode."""
         c, u = self.c, self
         w = c.frame.body.wrapper
@@ -780,7 +797,7 @@ class Undoer:
         bunch.oldYScroll = w.getYScrollPosition() if w else 0
         return bunch
     #@+node:ekr.20050315134017.6: *5* u.beforeChangeTree
-    def beforeChangeTree(self, p):
+    def beforeChangeTree(self, p: Position) -> None:
         u = self
         c = u.c
         w = c.frame.body.wrapper
@@ -790,26 +807,26 @@ class Undoer:
         bunch.oldTree = u.saveTree(p)
         return bunch
     #@+node:ekr.20050424161505.1: *5* u.beforeClearRecentFiles
-    def beforeClearRecentFiles(self):
+    def beforeClearRecentFiles(self) -> None:
         u = self
         p = u.c.p
         bunch = u.createCommonBunch(p)
         bunch.oldRecentFiles = g.app.config.recentFiles[:]
         return bunch
     #@+node:ekr.20050412080354: *5* u.beforeCloneNode
-    def beforeCloneNode(self, p):
+    def beforeCloneNode(self, p: Position) -> None:
         u = self
         bunch = u.createCommonBunch(p)
         return bunch
     #@+node:ekr.20050411193627.3: *5* u.beforeDeleteNode
-    def beforeDeleteNode(self, p):
+    def beforeDeleteNode(self, p: Position) -> None:
         u = self
         bunch = u.createCommonBunch(p)
         bunch.oldBack = p.back()
         bunch.oldParent = p.parent()
         return bunch
     #@+node:ekr.20050411193627.4: *5* u.beforeInsertNode
-    def beforeInsertNode(self, p, pasteAsClone=False, copiedBunchList=None):
+    def beforeInsertNode(self, p: Position, pasteAsClone: bool=False, copiedBunchList: Any=None) -> None:
         u = self
         if copiedBunchList is None:
             copiedBunchList = []
@@ -820,21 +837,21 @@ class Undoer:
             bunch.beforeTree = copiedBunchList
         return bunch
     #@+node:ekr.20050526131252: *5* u.beforeMark
-    def beforeMark(self, p, command):
+    def beforeMark(self, p: Position, command: str) -> None:
         u = self
         bunch = u.createCommonBunch(p)
         bunch.kind = 'mark'
         bunch.undoType = command
         return bunch
     #@+node:ekr.20050410110215: *5* u.beforeMoveNode
-    def beforeMoveNode(self, p):
+    def beforeMoveNode(self, p: Position) -> None:
         u = self
         bunch = u.createCommonBunch(p)
         bunch.oldN = p.childIndex()
         bunch.oldParent_v = p._parentVnode()
         return bunch
     #@+node:ekr.20080425060424.3: *5* u.beforeSort
-    def beforeSort(self, p, undoType, oldChildren, newChildren, sortChildren):
+    def beforeSort(self, p: Position, undoType: Any, oldChildren: Any, newChildren: Any, sortChildren: Any) -> None:
         """Create an undo node for sort operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -851,7 +868,7 @@ class Undoer:
         u.beads[u.bead:] = [bunch]
         return bunch
     #@+node:ekr.20050318085432.2: *5* u.createCommonBunch
-    def createCommonBunch(self, p):
+    def createCommonBunch(self, p: Position) -> None:
         """Return a bunch containing all common undo info.
         This is mostly the info for recreating an empty node at position p."""
         u = self
@@ -865,15 +882,15 @@ class Undoer:
     #@+node:ekr.20031218072017.3610: *4* u.canRedo & canUndo
     # Translation does not affect these routines.
 
-    def canRedo(self):
+    def canRedo(self) -> None:
         u = self
         return u.redoMenuLabel != "Can't Redo"
 
-    def canUndo(self):
+    def canUndo(self) -> None:
         u = self
         return u.undoMenuLabel != "Can't Undo"
     #@+node:ekr.20031218072017.3609: *4* u.clearUndoState
-    def clearUndoState(self):
+    def clearUndoState(self) -> None:
         """Clears then entire Undo state.
 
         All non-undoable commands should call this method."""
@@ -884,9 +901,17 @@ class Undoer:
         u.beads = []  # List of undo nodes.
         u.bead = -1  # Index of the present bead: -1:len(beads)
     #@+node:ekr.20031218072017.1490: *4* u.doTyping & helper
-    def doTyping(self, p, undo_type, oldText, newText,
-        newInsert=None, oldSel=None, newSel=None, oldYview=None,
-    ):
+    def doTyping(
+        self,
+        p: Position,
+        undo_type: Any,
+        oldText: Any,
+        newText: Any,
+        newInsert: Any=None,
+        oldSel: Any=None,
+        newSel: Any=None,
+        oldYview: Any=None,
+    ) -> None:
         """
         Save enough information to undo or redo a typing operation efficiently,
         that is, with the proper granularity.
@@ -1129,11 +1154,19 @@ class Undoer:
 
     setUndoTypingParams = doTyping
     #@+node:ekr.20050126081529: *5* u.recognizeStartOfTypingWord
-    def recognizeStartOfTypingWord(self,
-        old_lines, old_row, old_col, old_ch,
-        new_lines, new_row, new_col, new_ch,
-        prev_row, prev_col
-    ):
+    def recognizeStartOfTypingWord(
+        self,
+        old_lines: Any,
+        old_row: Any,
+        old_col: Any,
+        old_ch: Any,
+        new_lines: Any,
+        new_row: Any,
+        new_col: Any,
+        new_ch: Any,
+        prev_row: Any,
+        prev_col: Any,
+    ) -> None:
         """
         A potentially user-modifiable method that should return True if the
         typing indicated by the params starts a new 'word' for the purposes of
@@ -1155,7 +1188,7 @@ class Undoer:
         moved_cursor = new_row != prev_row or new_col != prev_col + 1
         return new_word_started or moved_cursor
     #@+node:ekr.20031218072017.3611: *4* u.enableMenuItems
-    def enableMenuItems(self):
+    def enableMenuItems(self) -> None:
         u = self
         frame = u.c.frame
         menu = frame.menu.getMenu("Edit")
@@ -1163,7 +1196,7 @@ class Undoer:
             frame.menu.enableMenu(menu, u.redoMenuLabel, u.canRedo())
             frame.menu.enableMenu(menu, u.undoMenuLabel, u.canUndo())
     #@+node:ekr.20110519074734.6094: *4* u.onSelect & helpers
-    def onSelect(self, old_p, p):
+    def onSelect(self, old_p: Any, p: Position) -> None:
 
         u = self
         if u.per_node_undo:
@@ -1172,7 +1205,7 @@ class Undoer:
             u.setIvarsFromVnode(p)
             u.setUndoTypes()
     #@+node:ekr.20110519074734.6096: *5* u.putIvarsToVnode
-    def putIvarsToVnode(self, p):
+    def putIvarsToVnode(self, p: Position) -> None:
 
         u, v = self, p.v
         assert self.per_node_undo
@@ -1184,7 +1217,7 @@ class Undoer:
             bunch[key] = getattr(u, key)
         v.undo_info = bunch
     #@+node:ekr.20110519074734.6095: *5* u.setIvarsFromVnode
-    def setIvarsFromVnode(self, p):
+    def setIvarsFromVnode(self, p: Position) -> None:
         u = self
         v = p.v
         assert self.per_node_undo
@@ -1192,7 +1225,7 @@ class Undoer:
         if hasattr(v, 'undo_info'):
             u.setIvarsFromBunch(v.undo_info)
     #@+node:ekr.20201127035748.1: *4* u.updateAfterTyping
-    def updateAfterTyping(self, p, w):
+    def updateAfterTyping(self, p: Position, w: Wrapper) -> None:
         """
         Perform all update tasks after changing body text.
 
@@ -1249,7 +1282,7 @@ class Undoer:
         w.setFocus()
     #@+node:ekr.20031218072017.2030: *3* u.redo
     @cmd('redo')
-    def redo(self, event=None):
+    def redo(self, event: Event=None) -> None:
         """Redo the operation undone by the last undo."""
         c, u = self.c, self
         if not c.p:
@@ -1277,11 +1310,11 @@ class Undoer:
         u.setUndoTypes()
     #@+node:ekr.20110519074734.6092: *3* u.redo helpers
     #@+node:ekr.20191213085226.1: *4*  u.reloadHelper (do nothing)
-    def redoHelper(self):
+    def redoHelper(self) -> None:
         """The default do-nothing redo helper."""
         pass
     #@+node:ekr.20201109080732.1: *4* u.redoChangeBody
-    def redoChangeBody(self):
+    def redoChangeBody(self) -> None:
         c, u, w = self.c, self, self.c.frame.body.wrapper
         # selectPosition causes recoloring, so don't do this unless needed.
         if c.p != u.p:  # #1333.
@@ -1304,7 +1337,7 @@ class Undoer:
         u.updateMarks('new')
         u.p.setDirty()
     #@+node:ekr.20201107150619.1: *4* u.redoChangeHeadline
-    def redoChangeHeadline(self):
+    def redoChangeHeadline(self) -> None:
         c, u = self.c, self
         # selectPosition causes recoloring, so don't do this unless needed.
         if c.p != u.p:  # #1333.
@@ -1316,28 +1349,28 @@ class Undoer:
         # This is required so.  Otherwise redraw will revert the change!
         c.frame.tree.setHeadline(u.p, u.newHead)
     #@+node:ekr.20050424170219: *4* u.redoClearRecentFiles
-    def redoClearRecentFiles(self):
+    def redoClearRecentFiles(self) -> None:
         u = self
         c = u.c
         rf = g.app.recentFilesManager
         rf.setRecentFiles(u.newRecentFiles[:])
         rf.createRecentFilesMenuItems(c)
     #@+node:ekr.20111005152227.15558: *4* u.redoCloneMarkedNodes
-    def redoCloneMarkedNodes(self):
+    def redoCloneMarkedNodes(self) -> None:
         u = self
         c = u.c
         c.selectPosition(u.p)
         c.cloneMarked()
         u.newP = c.p
     #@+node:ekr.20160502175557.1: *4* u.redoCopyMarkedNodes
-    def redoCopyMarkedNodes(self):
+    def redoCopyMarkedNodes(self) -> None:
         u = self
         c = u.c
         c.selectPosition(u.p)
         c.copyMarked()
         u.newP = c.p
     #@+node:ekr.20050412083057: *4* u.redoCloneNode
-    def redoCloneNode(self):
+    def redoCloneNode(self) -> None:
         u = self
         c = u.c
         cc = c.chapterController
@@ -1352,21 +1385,21 @@ class Undoer:
         c.selectPosition(u.newP)
         u.newP.setDirty()
     #@+node:ekr.20111005152227.15559: *4* u.redoDeleteMarkedNodes
-    def redoDeleteMarkedNodes(self):
+    def redoDeleteMarkedNodes(self) -> None:
         u = self
         c = u.c
         c.selectPosition(u.p)
         c.deleteMarked()
         c.selectPosition(u.newP)
     #@+node:EKR.20040526072519.2: *4* u.redoDeleteNode
-    def redoDeleteNode(self):
+    def redoDeleteNode(self) -> None:
         u = self
         c = u.c
         c.selectPosition(u.p)
         c.deleteOutline()
         c.selectPosition(u.newP)
     #@+node:ekr.20080425060424.9: *4* u.redoDemote
-    def redoDemote(self):
+    def redoDemote(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()
@@ -1382,7 +1415,7 @@ class Undoer:
         u.p.setDirty()
         c.setCurrentPosition(u.p)
     #@+node:ekr.20050318085432.6: *4* u.redoGroup
-    def redoGroup(self):
+    def redoGroup(self) -> None:
         """Process beads until the matching 'afterGroup' bead is seen."""
         c, u = self.c, self
         # Remember these values.
@@ -1419,19 +1452,19 @@ class Undoer:
             i, j = newSel
             c.frame.body.wrapper.setSelectionRange(i, j)
     #@+node:ekr.20050412085138.1: *4* u.redoHoistNode & redoDehoistNode
-    def redoHoistNode(self):
+    def redoHoistNode(self) -> None:
         c, u = self.c, self
         u.p.setDirty()
         c.selectPosition(u.p)
         c.hoist()
 
-    def redoDehoistNode(self):
+    def redoDehoistNode(self) -> None:
         c, u = self.c, self
         u.p.setDirty()
         c.selectPosition(u.p)
         c.dehoist()
     #@+node:ekr.20050412084532: *4* u.redoInsertNode
-    def redoInsertNode(self):
+    def redoInsertNode(self) -> None:
         u = self
         c = u.c
         cc = c.chapterController
@@ -1455,7 +1488,7 @@ class Undoer:
         u.newP.setDirty()
         c.selectPosition(u.newP)
     #@+node:ekr.20050526125801: *4* u.redoMark
-    def redoMark(self):
+    def redoMark(self) -> None:
         u = self
         c = u.c
         u.updateMarks('new')
@@ -1463,7 +1496,7 @@ class Undoer:
             u.p.setDirty()
             c.selectPosition(u.p)
     #@+node:ekr.20050411111847: *4* u.redoMove
-    def redoMove(self):
+    def redoMove(self) -> None:
         u = self
         c = u.c
         cc = c.chapterController
@@ -1488,7 +1521,7 @@ class Undoer:
         u.newP.setDirty()
         c.selectPosition(u.newP)
     #@+node:ekr.20050318085432.7: *4* u.redoNodeContents
-    def redoNodeContents(self):
+    def redoNodeContents(self) -> None:
         c, u = self.c, self
         w = c.frame.body.wrapper
         # selectPosition causes recoloring, so don't do this unless needed.
@@ -1511,7 +1544,7 @@ class Undoer:
         u.updateMarks('new')
         u.p.setDirty()
     #@+node:ekr.20080425060424.13: *4* u.redoPromote
-    def redoPromote(self):
+    def redoPromote(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()
@@ -1534,7 +1567,7 @@ class Undoer:
         u.p.setDirty()
         c.setCurrentPosition(u.p)
     #@+node:ekr.20080425060424.4: *4* u.redoSort
-    def redoSort(self):
+    def redoSort(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()
@@ -1543,7 +1576,7 @@ class Undoer:
         p.setAllAncestorAtFileNodesDirty()
         c.setCurrentPosition(p)
     #@+node:ekr.20050318085432.8: *4* u.redoTree
-    def redoTree(self):
+    def redoTree(self) -> None:
         """Redo replacement of an entire tree."""
         c, u = self.c, self
         u.p = self.undoRedoTree(u.oldTree, u.newTree)
@@ -1553,7 +1586,7 @@ class Undoer:
             i, j = u.newSel
             c.frame.body.wrapper.setSelectionRange(i, j)
     #@+node:EKR.20040526075238.5: *4* u.redoTyping
-    def redoTyping(self):
+    def redoTyping(self) -> None:
         u = self
         c = u.c
         current = c.p
@@ -1577,7 +1610,7 @@ class Undoer:
             w.setYScrollPosition(u.yview)
     #@+node:ekr.20031218072017.2039: *3* u.undo
     @cmd('undo')
-    def undo(self, event=None):
+    def undo(self, event: Event=None) -> None:
         """Undo the operation described by the undo parameters."""
         u = self
         c = u.c
@@ -1611,11 +1644,11 @@ class Undoer:
         u.setUndoTypes()
     #@+node:ekr.20110519074734.6093: *3* u.undo helpers
     #@+node:ekr.20191213085246.1: *4*  u.undoHelper
-    def undoHelper(self):
+    def undoHelper(self) -> None:
         """The default do-nothing undo helper."""
         pass
     #@+node:ekr.20201109080631.1: *4* u.undoChangeBody
-    def undoChangeBody(self):
+    def undoChangeBody(self) -> None:
         """
         Undo all changes to the contents of a node,
         including headline and body text, and marked bits.
@@ -1641,7 +1674,7 @@ class Undoer:
             c.frame.body.recolor(u.p)
         u.updateMarks('old')
     #@+node:ekr.20201107150041.1: *4* u.undoChangeHeadline
-    def undoChangeHeadline(self):
+    def undoChangeHeadline(self) -> None:
         """Undo a change to a node's headline."""
         c, u = self.c, self
         # selectPosition causes recoloring, so don't do this unless needed.
@@ -1653,14 +1686,14 @@ class Undoer:
         # This is required.  Otherwise c.redraw will revert the change!
         c.frame.tree.setHeadline(u.p, u.oldHead)
     #@+node:ekr.20050424170219.1: *4* u.undoClearRecentFiles
-    def undoClearRecentFiles(self):
+    def undoClearRecentFiles(self) -> None:
         u = self
         c = u.c
         rf = g.app.recentFilesManager
         rf.setRecentFiles(u.oldRecentFiles[:])
         rf.createRecentFilesMenuItems(c)
     #@+node:ekr.20111005152227.15560: *4* u.undoCloneMarkedNodes
-    def undoCloneMarkedNodes(self):
+    def undoCloneMarkedNodes(self) -> None:
         u = self
         next = u.p.next()
         assert next.h == 'Clones of marked nodes', (u.p, next.h)
@@ -1668,7 +1701,7 @@ class Undoer:
         u.p.setAllAncestorAtFileNodesDirty()
         u.c.selectPosition(u.p)
     #@+node:ekr.20050412083057.1: *4* u.undoCloneNode
-    def undoCloneNode(self):
+    def undoCloneNode(self) -> None:
         u = self
         c = u.c
         cc = c.chapterController
@@ -1679,7 +1712,7 @@ class Undoer:
         u.p.setDirty()
         c.selectPosition(u.p)
     #@+node:ekr.20160502175653.1: *4* u.undoCopyMarkedNodes
-    def undoCopyMarkedNodes(self):
+    def undoCopyMarkedNodes(self) -> None:
         u = self
         next = u.p.next()
         assert next.h == 'Copies of marked nodes', (u.p.h, next.h)
@@ -1687,7 +1720,7 @@ class Undoer:
         u.p.setAllAncestorAtFileNodesDirty()
         u.c.selectPosition(u.p)
     #@+node:ekr.20111005152227.15557: *4* u.undoDeleteMarkedNodes
-    def undoDeleteMarkedNodes(self):
+    def undoDeleteMarkedNodes(self) -> None:
         u = self
         c = u.c
         # Undo the deletes in reverse order
@@ -1703,7 +1736,7 @@ class Undoer:
         u.p.setAllAncestorAtFileNodesDirty()
         c.selectPosition(u.p)
     #@+node:ekr.20050412084055: *4* u.undoDeleteNode
-    def undoDeleteNode(self):
+    def undoDeleteNode(self) -> None:
         u = self
         c = u.c
         if u.oldBack:
@@ -1715,7 +1748,7 @@ class Undoer:
         u.p.setDirty()
         c.selectPosition(u.p)
     #@+node:ekr.20080425060424.10: *4* u.undoDemote
-    def undoDemote(self):
+    def undoDemote(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()
@@ -1733,7 +1766,7 @@ class Undoer:
         u.p.setAllAncestorAtFileNodesDirty()
         c.setCurrentPosition(u.p)
     #@+node:ekr.20050318085713: *4* u.undoGroup
-    def undoGroup(self):
+    def undoGroup(self) -> None:
         """Process beads until the matching 'beforeGroup' bead is seen."""
         c, u = self.c, self
         # Remember these values.
@@ -1773,21 +1806,21 @@ class Undoer:
             i, j = oldSel
             c.frame.body.wrapper.setSelectionRange(i, j)
     #@+node:ekr.20050412083244: *4* u.undoHoistNode & undoDehoistNode
-    def undoHoistNode(self):
+    def undoHoistNode(self) -> None:
         u = self
         c = u.c
         u.p.setDirty()
         c.selectPosition(u.p)
         c.dehoist()
 
-    def undoDehoistNode(self):
+    def undoDehoistNode(self) -> None:
         u = self
         c = u.c
         u.p.setDirty()
         c.selectPosition(u.p)
         c.hoist()
     #@+node:ekr.20050412085112: *4* u.undoInsertNode
-    def undoInsertNode(self):
+    def undoInsertNode(self) -> None:
         u = self
         c = u.c
         cc = c.chapterController
@@ -1809,7 +1842,7 @@ class Undoer:
                     v.setBodyString(bunch.body)
                     v.setHeadString(bunch.head)
     #@+node:ekr.20050526124906: *4* u.undoMark
-    def undoMark(self):
+    def undoMark(self) -> None:
         u = self
         c = u.c
         u.updateMarks('old')
@@ -1817,7 +1850,7 @@ class Undoer:
             u.p.setDirty()
             c.selectPosition(u.p)
     #@+node:ekr.20050411112033: *4* u.undoMove
-    def undoMove(self):
+    def undoMove(self) -> None:
 
         u = self
         c = u.c
@@ -1839,7 +1872,7 @@ class Undoer:
         u.p.setDirty()
         c.selectPosition(u.p)
     #@+node:ekr.20050318085713.1: *4* u.undoNodeContents
-    def undoNodeContents(self):
+    def undoNodeContents(self) -> None:
         """
         Undo all changes to the contents of a node,
         including headline and body text, and marked bits.
@@ -1863,7 +1896,7 @@ class Undoer:
             w.setYScrollPosition(u.oldYScroll)
         u.updateMarks('old')
     #@+node:ekr.20080425060424.14: *4* u.undoPromote
-    def undoPromote(self):
+    def undoPromote(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()  # The parent of the all the *promoted* nodes.
@@ -1886,13 +1919,18 @@ class Undoer:
         u.p.setAllAncestorAtFileNodesDirty()
         c.setCurrentPosition(u.p)
     #@+node:ekr.20031218072017.1493: *4* u.undoRedoText
-    def undoRedoText(self, p,
-        leading, trailing,  # Number of matching leading & trailing lines.
-        oldMidLines, newMidLines,  # Lists of unmatched lines.
-        oldNewlines, newNewlines,  # Number of trailing newlines.
-        tag="undo",  # "undo" or "redo"
-        undoType=None
-    ):
+    def undoRedoText(
+        self,
+        p: Position,
+        leading: Any,
+        trailing: Any,  # Number of matching leading & trailing lines.
+        oldMidLines: Any,
+        newMidLines: Any,  # Lists of unmatched lines.
+        oldNewlines: Any,
+        newNewlines: Any,  # Number of trailing newlines.
+        tag: str="undo",  # "undo" or "redo"
+        undoType: Any=None,
+    ) -> None:
         """Handle text undo and redo: converts _new_ text into _old_ text."""
         # newNewlines is unused, but it has symmetry.
         u = self
@@ -1930,7 +1968,7 @@ class Undoer:
         c.frame.body.recolor(p)
         w.seeInsertPoint()  # 2009/12/21
     #@+node:ekr.20050408100042: *4* u.undoRedoTree
-    def undoRedoTree(self, new_data, old_data):
+    def undoRedoTree(self, new_data: Any, old_data: Any) -> None:
         """Replace p and its subtree using old_data during undo."""
         # Same as undoReplace except uses g.Bunch.
         c, p, u = self.c, self.c.p, self
@@ -1945,7 +1983,7 @@ class Undoer:
         c.setBodyString(p, p.b)  # This is not a do-nothing.
         return p  # Nothing really changes.
     #@+node:ekr.20080425060424.5: *4* u.undoSort
-    def undoSort(self):
+    def undoSort(self) -> None:
         u = self
         c = u.c
         parent_v = u.p._parentVnode()
@@ -1954,7 +1992,7 @@ class Undoer:
         p.setAllAncestorAtFileNodesDirty()
         c.setCurrentPosition(p)
     #@+node:ekr.20050318085713.2: *4* u.undoTree
-    def undoTree(self):
+    def undoTree(self) -> None:
         """Redo replacement of an entire tree."""
         c, u = self.c, self
         u.p = self.undoRedoTree(u.newTree, u.oldTree)
@@ -1964,7 +2002,7 @@ class Undoer:
             i, j = u.oldSel
             c.frame.body.wrapper.setSelectionRange(i, j)
     #@+node:EKR.20040526090701.4: *4* u.undoTyping
-    def undoTyping(self):
+    def undoTyping(self) -> None:
         c, u = self.c, self
         w = c.frame.body.wrapper
         # selectPosition causes recoloring, so don't do this unless needed.
@@ -1985,7 +2023,7 @@ class Undoer:
             c.bodyWantsFocus()
             w.setYScrollPosition(u.yview)
     #@+node:ekr.20191213092304.1: *3* u.update_status
-    def update_status(self):
+    def update_status(self) -> None:
         """
         Update status after either an undo or redo:
         """
