@@ -1,6 +1,6 @@
 #@+leo-ver=5-thin
-#@+node:ekr.20150107090324.1: * @file cursesGui.py
-'''A minimal text-oriented gui.'''
+#@+node:ekr.20150107090324.1: * @file ../plugins/cursesGui.py
+"""A minimal text-oriented gui."""
 #@+at
 # Things not found in the GUI 'interface' classes (in leoFrame.py, leoGui.py, etc)
 # are labeled: # undoc: where the AttributeError comes from other implementations
@@ -9,23 +9,19 @@
 # pylint: disable=arguments-differ
 #@+<< imports >>
 #@+node:ekr.20150107090324.2: ** << imports >>
-# pylint: disable=wrong-import-order
-import leo.core.leoGlobals as g
-import leo.core.leoChapters as leoChapters
-import leo.core.leoGui as leoGui
-import leo.core.leoKeys as leoKeys
-import leo.core.leoFrame as leoFrame
-import leo.core.leoMenu as leoMenu
-import leo.core.leoNodes as leoNodes
-try:
-    import builtins # Python 3
-except ImportError:
-    import __builtin__ as builtins # Python 2.
 import os
-if 1: # Limit the disable to this statement.
-    # pylint: disable=no-member
-    get_input = input if g.isPython3 else builtins.raw_input
+from typing import Any, Callable
+from leo.core import leoGlobals as g
+from leo.core import leoChapters
+from leo.core import leoGui
+from leo.core import leoKeys
+from leo.core import leoFrame
+from leo.core import leoMenu
+from leo.core import leoNodes
+get_input = input
+
 #@-<< imports >>
+Widget = Any
 #@+<< TODO >>
 #@+node:ekr.20150107090324.3: ** << TODO >>
 #@@nocolor-node
@@ -53,13 +49,13 @@ if 1: # Limit the disable to this statement.
 #@+others
 #@+node:ekr.20150107090324.4: ** init
 def init():
-    ok = not g.app.gui and not g.app.unitTesting # Not Ok for unit testing!
+    ok = not g.app.gui and not g.unitTesting  # Not Ok for unit testing!
     if ok:
         g.app.gui = textGui()
         g.app.root = g.app.gui.createRootWindow()
         g.app.gui.finishCreate()
         g.plugin_signon(__name__)
-    elif g.app.gui and not g.app.unitTesting:
+    elif g.app.gui and not g.unitTesting:
         s = "Can't install text gui: previous gui installed"
         g.es_print(s, color="red")
     return ok
@@ -67,20 +63,19 @@ def init():
 def underline(s, idx):
     if idx < 0 or idx > len(s) - 1:
         return s
-    return s[: idx] + '&' + s[idx:]
+    return s[:idx] + '&' + s[idx:]
 #@+node:ekr.20150107090324.6: ** class textGui
 class textGui(leoGui.LeoGui):
     #@+others
     #@+node:ekr.20150107090324.7: *3* __init__
     def __init__(self):
-        leoGui.LeoGui.__init__(self, "text")
+        super().__init__("text")
         self.frames = []
         self.killed = False
         # TODO leoTkinterFrame finishCreate g.app.windowList.append(f) - use that?
     #@+node:ekr.20150107090324.8: *3* createKeyHandlerClass
     def createKeyHandlerClass(self, c):
-        # import leo.core.leoKeys as leoKeys
-            # Do this here to break a circular dependency.
+
         return leoKeys.KeyHandlerClass(c)
     #@+node:ekr.20150107090324.9: *3* createLeoFrame
     def createLeoFrame(self, c, title=None):
@@ -90,7 +85,7 @@ class textGui(leoGui.LeoGui):
         return ret
     #@+node:ekr.20150107090324.10: *3* createRootWindow
     def createRootWindow(self):
-        pass # N/A
+        pass  # N/A
     #@+node:ekr.20150107090324.11: *3* destroySelf
     def destroySelf(self):
         self.killed = True
@@ -105,7 +100,7 @@ class textGui(leoGui.LeoGui):
         pass
     #@+node:ekr.20150107090324.13: *3* isTextWidget
     def isTextWidget(self, w):
-        '''Return True if w is a Text widget suitable for text-oriented commands.'''
+        """Return True if w is a Text widget suitable for text-oriented commands."""
         return w and isinstance(w, leoFrame.StringTextWrapper)
     #@+node:ekr.20150107090324.14: *3* oops
     def oops(self):
@@ -117,7 +112,9 @@ class textGui(leoGui.LeoGui):
     def runMainLoop(self):
         self.text_run()
     #@+node:ekr.20150107090324.16: *3* runOpenFileDialog
-    def runOpenFileDialog(self, c, title, filetypes, defaultextension, multiple=False, startpath=None):
+    def runOpenFileDialog(self,
+        c, title, filetypes, defaultextension, multiple=False, startpath=None,
+    ) -> str:
         initialdir = g.app.globalOpenDir or g.os_path_abspath(os.getcwd())
         ret = get_input("Open which %s file (from %s?) > " % (repr(filetypes), initialdir))
         if multiple:
@@ -170,26 +167,33 @@ class TextFrame(leoFrame.LeoFrame):
     #@+others
     #@+node:ekr.20150107090324.22: *3* __init__
     def __init__(self, c, gui):
-        leoFrame.LeoFrame.__init__(self, c, gui)
+        super().__init__(c, gui)
         assert self.c == c
-        self.top = None ###
+        self.top = None
         self.ratio = self.secondary_ratio = 0.0
-        ### self.title = title # Per leoFrame.__init__
-    #@+node:ekr.20150107090324.23: *3* createFirstTreeNode
+    #@+node:ekr.20150107090324.23: *3* createFirstTreeNode (cursesGui.py)
     def createFirstTreeNode(self):
         c = self.c
+        #
+        # #1631: Initialize here, not in p._linkAsRoot.
+        c.hiddenRootNode.children = []
+        #
+        # #1817: Clear the gnxDict.
+        c.fileCommands.gnxDict = {}
+        #
         v = leoNodes.vnode(context=c)
         p = leoNodes.position(v)
         v.initHeadString("NewHeadline")
         # New in Leo 4.5: p.moveToRoot would be wrong:
         # the node hasn't been linked yet.
-        p._linkAsRoot(oldRoot=None)
+        p._linkAsRoot()
         # c.setRootPosition(p) # New in 4.4.2.
-        ### c.editPosition(p)
     #@+node:ekr.20150107090324.24: *3* deiconify
-    def deiconify(self): pass # N/A
+    def deiconify(self):
+        pass  # N/A
 
-    def lift(self): pass # N/A
+    def lift(self):
+        pass  # N/A
     #@+node:ekr.20150107090324.25: *3* destroySelf
     def destroySelf(self):
         pass
@@ -203,27 +207,30 @@ class TextFrame(leoFrame.LeoFrame):
         if f.body.use_chapters:
             c.chapterController = leoChapters.ChapterController(c)
         f.createFirstTreeNode()
-        ### c.initVersion()
         # (*after* setting self.log)
-        c.setLog() # writeWaitingLog hangs without this(!)
+        c.setLog()  # writeWaitingLog hangs without this(!)
         # So updateRecentFiles will update our menus.
         g.app.windowList.append(f)
     #@+node:ekr.20161118195504.1: *3* getFocus
     def getFocus(self):
         return None
     #@+node:ekr.20150107090324.27: *3* setInitialWindowGeometry
-    def setInitialWindowGeometry(self): pass # N/A
+    def setInitialWindowGeometry(self):
+        pass  # N/A
     #@+node:ekr.20150107090324.28: *3* setMinibufferBindings
     def setMinibufferBindings(self):
         pass
 
-    def setTopGeometry(self, w, h, x, y, adjustSize=True):
-        pass # N/A
+    def setTopGeometry(self, w, h, x, y):
+        pass  # N/A
     #@+node:ekr.20150107090324.29: *3* text_key
     def text_key(self):
-        c = self.c; k = c.k; w = self.body.bodyCtrl
+        c = self.c
+        k = c.k
+        w = self.body.bodyCtrl
         key = get_input('Keystroke > ')
-        if not key: return
+        if not key:
+            return
 
         class leoTypingEvent:
 
@@ -241,18 +248,20 @@ class TextFrame(leoFrame.LeoFrame):
         stroke = c.k.shortcutFromSetting(char)
         g.trace('char', repr(char), 'stroke', repr(stroke))
         e = leoTypingEvent(c, w, char, stroke)
-        k.masterKeyHandler(event=e) ## ,stroke=key)
+        k.masterKeyHandler(event=e)  ## ,stroke=key)
     #@+node:ekr.20150107090324.30: *3* update
-    def update(self): pass
+    def update(self):
+        pass
 
-    def resizePanesToRatio(self, ratio, ratio2): pass # N/A
+    def resizePanesToRatio(self, ratio: float, ratio2: float) -> None:
+        pass  # N/A
     #@-others
 #@+node:ekr.20150107090324.31: ** class textBody
 class textBody(leoFrame.LeoBody):
     #@+others
     #@+node:ekr.20150107090324.32: *3* __init__
     def __init__(self, frame, parentFrame):
-        leoFrame.LeoBody.__init__(self, frame, parentFrame)
+        super().__init__(frame, parentFrame)
         c = frame.c
         name = 'body'
         self.bodyCtrl = textBodyCtrl(c, name)
@@ -264,11 +273,11 @@ class textBody(leoFrame.LeoBody):
     def bind(self, bindStroke, callback):
         pass
     #@+node:ekr.20150107090324.34: *3* setEditorColors
-    # TODO Tkinter onBodyChanged undo call and many others. =(
+    def setEditorColors(self, bg, fg):
+        pass  # N/A
 
-    def setEditorColors(self, bg, fg): pass # N/A
-
-    def createBindings(self, w=None): pass
+    def createBindings(self, w=None):
+        pass
     #@+node:ekr.20150107090324.35: *3* text_show
     def text_show(self):
         w = self.bodyCtrl
@@ -280,7 +289,7 @@ class textBody(leoFrame.LeoBody):
 class textBodyCtrl(leoFrame.StringTextWrapper):
     pass
 #@+node:ekr.20150107090324.37: ** class textMenuCascade
-class textMenuCascade(object):
+class textMenuCascade:
     #@+others
     #@+node:ekr.20150107090324.38: *3* __init__
     def __init__(self, menu, label, underline):
@@ -295,7 +304,7 @@ class textMenuCascade(object):
         return ret
     #@-others
 #@+node:ekr.20150107090324.40: ** class textMenuEntry
-class textMenuEntry(object):
+class textMenuEntry:
     #@+others
     #@+node:ekr.20150107090324.41: *3* __init__
     def __init__(self, label, underline, accel, callback):
@@ -308,7 +317,7 @@ class textMenuEntry(object):
         return "%s %s" % (underline(self.label, self.underline), self.accel,)
     #@-others
 #@+node:ekr.20150107090324.43: ** class textMenuSep
-class textMenuSep(object):
+class textMenuSep:
     #@+others
     #@+node:ekr.20150107090324.44: *3* display
     def display(self):
@@ -321,27 +330,32 @@ class textLeoMenu(leoMenu.LeoMenu):
     def __init__(self, frame):
         self.entries = []
         self.c = frame.c
-        # Init the base class
-        leoMenu.LeoMenu.__init__(self, frame)
+        super().__init__(frame)
     #@+node:ekr.20150107090324.47: *3* createMenuBar
     def createMenuBar(self, frame):
         g.trace(frame.c)
         self._top_menu = textLeoMenu(frame)
         self.createMenusFromTables()
     #@+node:ekr.20150107090324.48: *3* new_menu
-    def new_menu(self, parent, tearoff=0, label=''):
+    def new_menu(self, parent: Widget, tearoff: int=0, labe: str=''):
         if tearoff:
             raise NotImplementedError(repr(tearoff))
         menu = textLeoMenu(parent or self.frame)
         menu.entries = []
         return menu
     #@+node:ekr.20150107090324.49: *3* add_cascade
-    def add_cascade(self, parent, label, menu, underline):
+    def add_cascade(self, parent: Any, label: str, menu: Any, underline: int):
         if parent is None:
             parent = self._top_menu
         parent.entries.append(textMenuCascade(menu, label, underline,))
-    #@+node:ekr.20150107090324.50: *3* add_command
-    def add_command(self, **keys):
+    #@+node:ekr.20150107090324.50: *3* add_command (cursesGui.py)
+    def add_command(self, menu: Widget,
+        accelerator: str='',
+        command: Callable=None,
+        commandName: str=None,
+        label: str=None,
+        underline: int=0,
+    ) -> None:
         # ?
         # underline - Offset into label. For those who memorised Alt, F, X rather than Alt+F4.
         # accelerator - For display only; these are implemented by Leo's key handling.
@@ -350,10 +364,13 @@ class textLeoMenu(leoMenu.LeoMenu):
         def doNothingCallback():
             pass
 
-        label = keys.get('label') or 'no label'
-        underline = keys.get('underline') or 0
-        accelerator = keys.get('accelerator') or ''
-        command = keys.get('command') or doNothingCallback
+        if not command:
+            command = doNothingCallback
+
+        # label = keys.get('label') or 'no label'
+        # underline = keys.get('underline') or 0
+        # accelerator = keys.get('accelerator') or ''
+        # command = keys.get('command') or doNothingCallback
         entry = textMenuEntry(label, underline, accelerator, command)
         menu.entries.append(entry)
     #@+node:ekr.20150107090324.51: *3* add_separator
@@ -372,7 +389,8 @@ class textLeoMenu(leoMenu.LeoMenu):
             g.pr(len(last_menu.entries), ')', '[Prev]')
             which = get_input('Which menu entry? > ')
             which = which.strip()
-            if not which: continue
+            if not which:
+                continue
             try:
                 n = int(which)
             except ValueError:
@@ -401,9 +419,11 @@ class textLog(leoFrame.LeoLog):
     #@+node:ekr.20150107090324.58: *3* createControl
     # < < HACK Quiet, oops. >>
 
-    def createControl(self, parentFrame): pass
+    def createControl(self, parentFrame):
+        pass
 
-    def setFontFromConfig(self): pass # N/A
+    def setFontFromConfig(self):
+        pass  # N/A
     #@+node:ekr.20150107090324.68: *3* finishCreate
     def finishCreate(self):
         pass
@@ -414,34 +434,30 @@ class textLog(leoFrame.LeoLog):
     def putnl(self, tabName='log'):
         g.pr('')
     #@+node:ekr.20150107090324.59: *3* setColorFromConfig
-    def setColorFromConfig(self): pass
+    def setColorFromConfig(self):
+        pass
     #@+node:ekr.20150107090324.55: *3* setTabBindings
     def setTabBindings(self, tabName):
         pass
     #@-others
 #@+node:ekr.20150107090324.60: ** class textTree
 class textTree(leoFrame.LeoTree):
-    # undoc: k.makeAllBindings ; nullTree
     #@+others
     #@+node:ekr.20150107090324.61: *3* setBindings
     def setBindings(self):
         pass
     #@+node:ekr.20150107090324.62: *3* begin/endUpdate & redraw/now
-    def redraw(self, p=None, scroll=True, forceDraw=False):
+    def redraw(self, p=None):
         self.text_draw_tree()
-        
-    redraw_now = redraw
 
-    # def redraw_now(self, p=None, scroll=True, forceDraw=False):
-        # if forceDraw:
-            # self.redraw()
+    redraw_now = redraw
     #@+node:ekr.20150107090324.63: *3* endUpdate
     #@+node:ekr.20150107090324.64: *3* __init__
     def __init__(self, frame):
         # undoc: openWithFileName -> treeWantsFocus -> c.frame.tree.canvas
         self.c = frame.c
         self.canvas = None
-        leoFrame.LeoTree.__init__(self, frame)
+        super().__init__(frame)
     #@+node:ekr.20150107090324.65: *3* select
     def select(self, p, scroll=True):
         # TODO Much more here: there's four hooks and all sorts of other things called in the TK version.
@@ -454,7 +470,7 @@ class textTree(leoFrame.LeoTree):
         # and something to do with undo?
     #@+node:ekr.20150107090324.66: *3* editLabel & edit_widget
     def editLabel(self, v, selectAll=False, selection=None):
-        pass # N/A?
+        pass  # N/A?
 
     def edit_widget(self, p):
         return None
