@@ -1,5 +1,5 @@
 #@+leo-ver=5-thin
-#@+node:tbrown.20090206153748.1: * @file graphcanvas.py
+#@+node:tbrown.20090206153748.1: * @file ../plugins/graphcanvas.py
 """
 Provides a widget for displaying graphs (networks) in Leo.
 
@@ -23,56 +23,52 @@ make sense to focus on pydot.
 
 #@+<< imports >>
 #@+node:bob.20110119123023.7392: ** << imports >> graphcanvas
-import leo.core.leoGlobals as g
-import leo.core.leoPlugins as leoPlugins
 
 from math import atan2, sin, cos
-# import time
 import os
 import tempfile
+from typing import Any, Dict
+import urllib.request as urllib
 
-# pylint: disable=no-name-in-module
-if g.isPython3:
-    import urllib.request as urllib
-else:
-    import urllib2 as urllib
-
-# from xml.sax.saxutils import quoteattr
-
+from leo.core import leoGlobals as g
+from leo.core import leoPlugins
+from leo.core.leoQt import QtConst, QtCore, QtGui, QtWidgets, uic
+from leo.core.leoQt import KeyboardModifier
+# Third-party imports
 try:
+    # pylint: disable=import-error
+        # These are optional.
     import pydot
     import dot_parser
     assert dot_parser
+    pygraphviz = None
 except Exception:
     pydot = None
-
-# Fail gracefully if the gui is not qt.
-g.assertUi('qt')
-from leo.core.leoQt import QtConst, QtCore, QtGui, QtWidgets, uic
-
-pygraphviz = None
-if not pydot:
     try:
-        import pygraphviz
+        import pygraphviz  # type:ignore
     except ImportError:
         pygraphviz = None
+#
+# Fail fast, right after all imports.
+g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
+
 #@-<< imports >>
 c_db_key = '_graph_canvas_gnx'
 #@+others
 #@+node:bob.20110119123023.7393: ** init
-def init ():
-    '''Return True if the plugin has loaded successfully.'''
+def init():
+    """Return True if the plugin has loaded successfully."""
     if g.app.gui.guiName() != "qt":
         return False
     g.visit_tree_item.add(colorize_headlines_visitor)
-    g.registerHandler('after-create-leo-frame',onCreate)
+    g.registerHandler('after-create-leo-frame', onCreate)
     # can't use before-create-leo-frame because Qt dock's not ready
     g.loadOnePlugin("backlink.py")
     g.plugin_signon(__name__)
     return True
 #@+node:bob.20110121094946.3410: ** colorize_headlines_visitor
-def colorize_headlines_visitor(c,p, item):
-    '''Item is a QTreeWidgetItem.'''
+def colorize_headlines_visitor(c, p, item):
+    """Item is a QTreeWidgetItem."""
     if '_bklnk' in p.v.u:
         # f = item.font(0)
         # f.setItalic(True)
@@ -87,10 +83,11 @@ def colorize_headlines_visitor(c,p, item):
             f.setBold(True)
     raise leoPlugins.TryNext
 #@+node:bob.20110119123023.7394: ** onCreate
-def onCreate (tag, keys):
+def onCreate(tag, keys):
 
     c = keys.get('c')
-    if not c: return
+    if not c:
+        return
     graphcanvasController(c)
     if hasattr(c, 'db') and c_db_key in c.db:
         gnx = c.db[c_db_key]
@@ -121,13 +118,13 @@ def toggle_autoload(event):
         c.db[c_db_key] = str(c.p.v.gnx)
         g.es('Graph for current node will be autoloaded')
 #@+node:bob.20110119123023.7395: ** class graphcanvasUI
-class graphcanvasUI(QtWidgets.QWidget):
+class graphcanvasUI(QtWidgets.QWidget):  # type:ignore
     #@+others
     #@+node:bob.20110119123023.7396: *3* __init__
     def __init__(self, owner=None):
 
         self.owner = owner
-        QtWidgets.QWidget.__init__(self)
+        super().__init__()
         uiPath = g.os_path_join(g.app.leoDir,
             'plugins', 'GraphCanvas', 'GraphCanvas.ui')
         # change directory for this to work
@@ -135,7 +132,7 @@ class graphcanvasUI(QtWidgets.QWidget):
         try:
             os.chdir(g.os_path_join(g.computeLeoDir(), ".."))
             form_class, base_class = uic.loadUiType(uiPath)
-            self.owner.c.frame.log.createTab('Graph', widget = self)
+            self.owner.c.frame.log.createTab('Graph', widget=self)
             self.UI = form_class()
             self.UI.setupUi(self)
         finally:
@@ -144,7 +141,7 @@ class graphcanvasUI(QtWidgets.QWidget):
         self.canvasView = GraphicsView(self.owner, self.canvas)
         self.canvasView.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)
         self.UI.canvasFrame.addWidget(self.canvasView)
-        self.canvasView.setSceneRect(0,0,300,300)
+        self.canvasView.setSceneRect(0, 0, 300, 300)
         self.canvasView.setRenderHints(QtGui.QPainter.Antialiasing)
         u = self.UI
         o = self.owner
@@ -194,19 +191,19 @@ class graphcanvasUI(QtWidgets.QWidget):
         self.canvasView.current_scale = 0
     #@-others
 #@+node:bob.20110119123023.7397: ** class GraphicsView
-class GraphicsView(QtWidgets.QGraphicsView):
+class GraphicsView(QtWidgets.QGraphicsView):  # type:ignore
     #@+others
     #@+node:bob.20110119123023.7398: *3* __init__
     def __init__(self, glue, *args, **kargs):
         self.glue = glue
         self.current_scale = 0
-        QtWidgets.QGraphicsView.__init__(self, *args)
-    #@+node:tbrown.20110122085529.15399: *3* wheelEvent
+        super().__init__(*args)
+    #@+node:tbrown.20110122085529.15399: *3* wheelEvent (graphcanvas.py)
     def wheelEvent(self, event):
 
-        if int(event.modifiers() & QtConst.ControlModifier):
+        if int(event.modifiers() & KeyboardModifier.ControlModifier):
 
-            scale = 1.+0.1*(event.delta() / 120)
+            scale = 1. + 0.1 * (event.delta() / 120)
 
             self.scale(scale, scale)
 
@@ -223,7 +220,7 @@ class GraphicsView(QtWidgets.QGraphicsView):
         self.glue.newNode(pnt=self.mapToScene(event.pos()))
     #@-others
 #@+node:tbrown.20110413094721.24681: ** class GetImage
-class GetImage(object):
+class GetImage:
     """Image handling functions"""
 
     @staticmethod
@@ -260,38 +257,25 @@ class GetImage(object):
         if '//' not in src or src.startswith('file://'):
             testpath = src
             if '//' in testpath:
-                testpath = testpath.split('//',1)[-1]
-
+                testpath = testpath.split('//', 1)[-1]
+            #
             # file on local file system
             testpath = g.os_path_finalize_join(path, testpath)
             if g.os_path_exists(testpath):
                 return QtWidgets.QGraphicsPixmapItem(QtGui.QPixmap(testpath))
-
+            #
             # explicit file://, but no such file exists
             if src.startswith('file://'):
-                if fail_ok:
-                    return None
-                else:
-                    return GetImage._no_image()
-
+                return None if fail_ok else GetImage._no_image()
+        #
         # no explict file://, so try other protocols
-
-        if '//' not in src:
-            testpath = 'http://%s' % src
-        else:
-            testpath = src
-
+        testpath = src if '//' in src else 'http://%s' % (src)
         data = GetImage.get_url(testpath)
-
         if data:
             img = QtGui.QPixmap()
             if img.loadFromData(data):
                 return QtWidgets.QGraphicsPixmapItem(img)
-
-        if fail_ok:
-            return None
-
-        return GetImage._no_image()
+        return None if fail_ok else GetImage._no_image()
 
     @staticmethod
     def get_url(url):
@@ -300,20 +284,18 @@ class GetImage(object):
             response = urllib.urlopen(url)
         except urllib.URLError:  # hopefully not including redirection
             return False
-
         return response.read()
 
     @staticmethod
     def _no_image():
         """return QGraphicsPixmapItem with "No Image" image loaded"""
         testpath = g.os_path_abspath(g.os_path_join(
-            g.app.loadDir,'../plugins/GraphCanvas/no_image.png'))
+            g.app.loadDir, '../plugins/GraphCanvas/no_image.png'))
         return QtWidgets.QGraphicsPixmapItem(QtGui.QPixmap(testpath))
-
 #@+node:tbrown.20110407091036.17531: ** class nodeBase
-class nodeBase(QtWidgets.QGraphicsItemGroup):
+class nodeBase(QtWidgets.QGraphicsItemGroup):  # type:ignore
 
-    node_types = {}
+    node_types: Dict[str, Any] = {}
 
     @classmethod
     def make_node(cls, owner, node, ntype):
@@ -321,7 +303,7 @@ class nodeBase(QtWidgets.QGraphicsItemGroup):
 
     def __init__(self, owner, node, *args, **kargs):
 
-        QtWidgets.QGraphicsItemGroup.__init__(self, *args, **kargs)
+        super().__init__(*args, **kargs)
         self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable)
 
         self.owner = owner
@@ -354,7 +336,7 @@ class nodeBase(QtWidgets.QGraphicsItemGroup):
     #@+node:tbrown.20110407091036.17538: *3* focusOutEvent
     def focusOutEvent(self, event):
         QtWidgets.QGraphicsItemGroup.focusOutEvent(self, event)
-        self.bg.setBrush(QtGui.QBrush(QtGui.QColor(200,240,200)))
+        self.bg.setBrush(QtGui.QBrush(QtGui.QColor(200, 240, 200)))
         g.es("focusOutEvent")
     #@-others
 #@+node:tbrown.20110407091036.17533: ** class nodeRect
@@ -362,15 +344,15 @@ class nodeRect(nodeBase):
     """text with shape behind it node type"""
 
     def __init__(self, *args, **kargs):
-        nodeBase.__init__(self, *args, **kargs)
+        super().__init__(*args, **kargs)
 
         self.text = self.text_item()
         # .text must be first for nodeComment, see its bg_item()
         self.bg = self.bg_item()
         if g.app.config.getBool("color-theme-is-dark"):
-            bgcolor = QtGui.QColor(30,50,30)
+            bgcolor = QtGui.QColor(30, 50, 30)
         else:
-            bgcolor = QtGui.QColor(200,240,200)
+            bgcolor = QtGui.QColor(200, 240, 200)
         self.bg.setBrush(QtGui.QBrush(bgcolor))
 
         self.setZValue(20)
@@ -386,7 +368,7 @@ class nodeRect(nodeBase):
 
     def bg_item(self):
         """return a canvas item for the shape in the background"""
-        return QtWidgets.QGraphicsRectItem(-2,+2,30,20)
+        return QtWidgets.QGraphicsRectItem(-2, +2, 30, 20)
 
     def text_item(self):
         """return a canvas item for the text in the foreground"""
@@ -410,8 +392,8 @@ class nodeRect(nodeBase):
         self.text.setPlainText(self.get_text())
 
         self.bg.setRect(-2, +2,
-            self.text.document().size().width()+4,
-            self.text.document().size().height()-2)
+            self.text.document().size().width() + 4,
+            self.text.document().size().height() - 2)
 
 nodeBase.node_types[nodeRect.__name__] = nodeRect
 #@+node:tbrown.20110413094721.20406: ** class nodeNone
@@ -419,7 +401,7 @@ class nodeNone(nodeBase):
     """text with shape behind it node type"""
 
     def __init__(self, *args, **kargs):
-        nodeBase.__init__(self, *args, **kargs)
+        super().__init__(*args, **kargs)
 
         self.text = self.text_item()
 
@@ -454,13 +436,13 @@ class nodeEllipse(nodeRect):
 
     def bg_item(self):
         """return a canvas item for the shape in the background"""
-        return QtWidgets.QGraphicsEllipseItem(-5,+5,30,20)
+        return QtWidgets.QGraphicsEllipseItem(-5, +5, 30, 20)
 
     def do_update(self):
-        marginX = self.text.document().size().width()/2
+        marginX = self.text.document().size().width() / 2
         # marginY = self.text.document().size().height()/2
         self.bg.setRect(-marginX, 0,
-            self.text.document().size().width()*2,
+            self.text.document().size().width() * 2,
             self.text.document().size().height())
 
 nodeBase.node_types[nodeEllipse.__name__] = nodeEllipse
@@ -481,11 +463,11 @@ class nodeDiamond(nodeRect):
 
     def do_update(self):
         poly = QtGui.QPolygonF()
-        marginX = self.text.document().size().width()/2
-        marginY = self.text.document().size().height()/2
+        marginX = self.text.document().size().width() / 2
+        marginY = self.text.document().size().height() / 2
         poly.append(QtCore.QPointF(-marginX, marginY))
-        poly.append(QtCore.QPointF(marginX, 3*marginY))
-        poly.append(QtCore.QPointF(3*marginX, marginY))
+        poly.append(QtCore.QPointF(marginX, 3 * marginY))
+        poly.append(QtCore.QPointF(3 * marginX, marginY))
         poly.append(QtCore.QPointF(marginX, -marginY))
         self.bg.setPolygon(poly)
 
@@ -520,8 +502,8 @@ class nodeComment(nodeRect):
         self._set_text(self.text)
 
         self.bg.setRect(-2, +2,
-            self.text.document().size().width()+4,
-            self.text.document().size().height()-2)
+            self.text.document().size().width() + 4,
+            self.text.document().size().height() - 2)
 
         self.setToolTip(self.node.h)
 
@@ -530,18 +512,14 @@ nodeBase.node_types[nodeComment.__name__] = nodeComment
 class nodeTable(nodeRect):
 
     def __init__(self, *args, **kargs):
-        nodeRect.__init__(self, *args, **kargs)
-
+        super().__init__(*args, **kargs)
         # can't load children here, because we don't know where we are yet
-
         self.updating = False
 
     def do_update(self):
         nodeRect.do_update(self)
-
         if self.updating:
             return
-
         what = []
         dy = self.text.document().size().height()
         for n, child in enumerate(self.node.children):
@@ -550,7 +528,7 @@ class nodeTable(nodeRect):
                     child.u['_bklnk'] = {}
                 if 'x' not in child.u['_bklnk']:
                     child.u['_bklnk']['x'] = self.node.u['_bklnk']['x'] + 16
-                    child.u['_bklnk']['y'] = self.node.u['_bklnk']['y'] + dy * (n+1)
+                    child.u['_bklnk']['y'] = self.node.u['_bklnk']['y'] + dy * (n + 1)
                 child.u['_bklnk']['type'] = nodeRect.__name__
                 what.append(child)
 
@@ -565,13 +543,13 @@ class nodeTable(nodeRect):
 
         nodeRect.mouseMoveEvent(self, event)
 
-        dx, dy = self.x()-ox, self.y()-oy
+        dx, dy = self.x() - ox, self.y() - oy
 
         for n, child in enumerate(self.node.children):
             if child in self.owner.nodeItem:
                 childItem = self.owner.nodeItem[child]
-                childItem.setX(childItem.x()+dx)
-                childItem.setY(childItem.y()+dy)
+                childItem.setX(childItem.x() + dx)
+                childItem.setY(childItem.y() + dy)
                 self.owner.newPos(childItem, None)
 
 
@@ -580,7 +558,7 @@ nodeBase.node_types[nodeTable.__name__] = nodeTable
 class nodeImage(nodeBase):
 
     def __init__(self, *args, **kargs):
-        nodeBase.__init__(self, *args, **kargs)
+        super().__init__(*args, **kargs)
 
         self.bg = self.bg_item()
 
@@ -606,7 +584,7 @@ class nodeImage(nodeBase):
 
 nodeBase.node_types[nodeImage.__name__] = nodeImage
 #@+node:bob.20110121161547.3424: ** class linkItem
-class linkItem(QtWidgets.QGraphicsItemGroup):
+class linkItem(QtWidgets.QGraphicsItemGroup):  # type:ignore
     """Node on the canvas"""
     #@+others
     #@+node:bob.20110119123023.7405: *3* __init__
@@ -616,10 +594,8 @@ class linkItem(QtWidgets.QGraphicsItemGroup):
 
         pass glue object and let it key nodeItems to leo nodes
         """
-        # pylint: disable=keyword-arg-before-vararg
-            # putting *args first is invalid in Python 2.x.
         self.glue = glue
-        QtWidgets.QGraphicsItemGroup.__init__(self)
+        super().__init__()
         self.line = QtWidgets.QGraphicsLineItem(*args)
 
         pen = QtGui.QPen()
@@ -630,8 +606,8 @@ class linkItem(QtWidgets.QGraphicsItemGroup):
             pen.setWidth(2)
         else:
             self.setZValue(0)
-            pen.setColor(QtGui.QColor(240,240,240))
-            pen.setWidth(2) # (0.5)
+            pen.setColor(QtGui.QColor(240, 240, 240))
+            pen.setWidth(2)  # (0.5)
 
         self.line.setPen(pen)
         self.addToGroup(self.line)
@@ -639,9 +615,9 @@ class linkItem(QtWidgets.QGraphicsItemGroup):
         self.head = QtWidgets.QGraphicsPolygonItem()
 
         if hierarchyLink:
-            self.head.setBrush(QtGui.QBrush(QtGui.QColor(180,180,180)))
+            self.head.setBrush(QtGui.QBrush(QtGui.QColor(180, 180, 180)))
         else:
-            self.head.setBrush(QtGui.QBrush(QtGui.QColor(0,0,0)))
+            self.head.setBrush(QtGui.QBrush(QtGui.QColor(0, 0, 0)))
 
         self.head.setPen(QtGui.QPen(QtConst.NoPen))
         self.addToGroup(self.head)
@@ -650,52 +626,52 @@ class linkItem(QtWidgets.QGraphicsItemGroup):
         QtWidgets.QGraphicsItemGroup.mousePressEvent(self, event)
         self.glue.pressLink(self, event)
     #@+node:bob.20110119123023.7407: *3* setLine
-    def setLine(self, x0, y0, x1, y1, hierarchyLink = False):
+    def setLine(self, x0, y0, x1, y1, hierarchyLink=False):
 
         self.line.setLine(x0, y0, x1, y1)
 
-        x,y = x1-(x1-x0)/3., y1-(y1-y0)/3.
+        x, y = x1 - (x1 - x0) / 3., y1 - (y1 - y0) / 3.
 
         if not hierarchyLink:
             r = 12.
         else:
             r = 6.
 
-        a = atan2(y1-y0, x1-x0)
+        a = atan2(y1 - y0, x1 - x0)
         w = 2.79252680
         pts = [
             QtCore.QPointF(x, y),
-            QtCore.QPointF(x+r*cos(a+w), y+r*sin(a+w)),
-            QtCore.QPointF(x+r*cos(a-w), y+r*sin(a-w)),
+            QtCore.QPointF(x + r * cos(a + w), y + r * sin(a + w)),
+            QtCore.QPointF(x + r * cos(a - w), y + r * sin(a - w)),
             # QtCore.QPointF(x, y),
         ]
         self.head.setPolygon(QtGui.QPolygonF(pts))
     #@-others
 #@+node:bob.20110119123023.7408: ** class graphcanvasController
-class graphcanvasController(object):
+class graphcanvasController:
     """Display and edit links in leo"""
     #@+others
     #@+node:bob.20110119123023.7409: *3* __init__ & reloadSettings (graphcanvasController)
-    def __init__ (self,c):
+    def __init__(self, c):
 
         self.c = c
         self.c.graphcanvasController = self
-        self.selectPen = QtGui.QPen(QtGui.QColor(255,0,0))
+        self.selectPen = QtGui.QPen(QtGui.QColor(255, 0, 0))
         self.selectPen.setWidth(2)
         self.ui = graphcanvasUI(self)
-        g.registerHandler('headkey2', lambda a,b: self.do_update())
+        g.registerHandler('headkey2', lambda a, b: self.do_update())
         g.registerHandler("select2", self.onSelect2)
         # g.registerHandler('open2', self.loadLinks)
             # already missed initial 'open2' because of after-create-leo-frame, so
             # self.loadLinksInt()
         self.initIvars()
         self.reloadSettings()
-        
+
     def reloadSettings(self):
         c = self.c
         c.registerReloadSettings(self)
         self.graph_manual_layout = \
-            c.config.getBool('graph-manual-layout',default=False)
+            c.config.getBool('graph-manual-layout', default=False)
     #@+node:bob.20110119123023.7410: *3* initIvars
     def initIvars(self):
         """initialize, called by __init__ and clear"""
@@ -718,7 +694,7 @@ class graphcanvasController(object):
                 ('dot', lambda: self.layout('dot')),
                 ('dot LR', lambda: self.layout('dot LR')),
             ]
-        elif pydot:
+        if pydot:
             return [
                 ('PyDot:', lambda: None),
                 ('neato', lambda: self.layout('neato')),
@@ -729,18 +705,17 @@ class graphcanvasController(object):
                 ('osage', lambda: self.layout('osage')),
                 ('sfdp', lambda: self.layout('sfdp')),
         ]
-        else:
-            return [('install pygraphviz or pydot for layouts', lambda: None)]
+        return [('install pygraphviz or pydot for layouts', lambda: None)]
 
     #@+node:tbrown.20110122085529.15403: *3* layout
     def layout(self, type_):
 
         if pygraphviz:
-            G = pygraphviz.AGraph(strict=False,directed=True)
+            G = pygraphviz.AGraph(strict=False, directed=True)
             if type_ == 'dot LR':
-                G.graph_attr['rankdir']='LR'
+                G.graph_attr['rankdir'] = 'LR'
             type_ = type_.split()[0]
-            G.graph_attr['ranksep']='0.125'
+            G.graph_attr['ranksep'] = '0.125'
         elif pydot:
             G = pydot.Dot('graphname', graph_type='digraph')
             if type_ == 'dot LR':
@@ -756,20 +731,20 @@ class graphcanvasController(object):
                     (to, to.gnx),
             )
             elif pydot:
-                G.add_edge(pydot.Edge( from_.gnx, to.gnx ))
+                G.add_edge(pydot.Edge(from_.gnx, to.gnx))
 
         for i in self.nodeItem:
             if pygraphviz:
-                G.add_node( (i, i.gnx) )
+                G.add_node((i, i.gnx))
             elif pydot:
-                gnode = pydot.Node( i.gnx)
+                gnode = pydot.Node(i.gnx)
                 # rect = self.nodeItem[i].boundingRect()
                 G.add_node(gnode)
                 for child in i.children:
                     key = (i, child)
                     if key not in self.hierarchyLinkItem or child not in self.nodeItem:
                         continue
-                    G.add_edge(pydot.Edge( i.gnx, child.gnx ))
+                    G.add_edge(pydot.Edge(i.gnx, child.gnx))
         if pygraphviz:
             G.layout(prog=type_)
         elif pydot:
@@ -779,8 +754,8 @@ class graphcanvasController(object):
 
         for i in self.nodeItem:
             if pygraphviz:
-                gn = G.get_node( (i, i.gnx) )
-                x,y = map(float, gn.attr['pos'].split(','))
+                gn = G.get_node((i, i.gnx))
+                x, y = map(float, gn.attr['pos'].split(','))
                 i.u['_bklnk']['x'] = x
                 i.u['_bklnk']['y'] = -y
                 self.nodeItem[i].setPos(x, -y)
@@ -788,20 +763,20 @@ class graphcanvasController(object):
             elif pydot:
                 lst = G.get_node(''.join(['"', i.gnx, '"']))
                 if lst:
-                    x,y = map(float, lst[0].get_pos().strip('"').split(','))
+                    x, y = map(float, lst[0].get_pos().strip('"').split(','))
                     i.u['_bklnk']['x'] = x
                     i.u['_bklnk']['y'] = -y
                     self.nodeItem[i].setPos(x, -y)
                     self.nodeItem[i].do_update()
         if pydot:
-            x,y,width,height = map(float, G.get_bb().strip('"').split(','))
-            self.ui.canvasView.setSceneRect(self.ui.canvas.sceneRect().adjusted(x,y,width,height))
+            x, y, width, height = map(float, G.get_bb().strip('"').split(','))
+            self.ui.canvasView.setSceneRect(self.ui.canvas.sceneRect().adjusted(x, y, width, height))
         self.do_update(adjust=False)
         self.center_graph()
         # self.ui.canvasView.centerOn(self.ui.canvas.sceneRect().center())
         # self.ui.canvasView.fitInView(self.ui.canvas.sceneRect(), QtConst.KeepAspectRatio)
     #@+node:bob.20110119133133.3353: *3* loadGraph
-    def loadGraph(self, what='node', create = True, pnt=None):
+    def loadGraph(self, what='node', create=True, pnt=None):
 
         if what == 'sibs':
             collection = self.c.currentPosition().self_and_siblings()
@@ -863,18 +838,18 @@ class graphcanvasController(object):
             if 'tcolor' in node.u['_bklnk']:
                 node_obj.set_text_color(node.u['_bklnk']['tcolor'])
 
-            x,y = 0,0
+            x, y = 0, 0
             if pnt:
-                x,y = pnt.x(), pnt.y()
+                x, y = pnt.x(), pnt.y()
                 node.u['_bklnk']['x'] = x
                 node.u['_bklnk']['y'] = y
             elif 'x' in node.u['_bklnk']:
-                x,y = node.u['_bklnk']['x'], node.u['_bklnk']['y']
+                x, y = node.u['_bklnk']['x'], node.u['_bklnk']['y']
             else:
                 node.u['_bklnk']['x'] = x
                 node.u['_bklnk']['y'] = y
 
-            node_obj.setPos(x,y)
+            node_obj.setPos(x, y)
             self.ui.canvas.addItem(node_obj)
 
         self.do_update()
@@ -904,7 +879,7 @@ class graphcanvasController(object):
                 # none added, or doing just one round
                 break
     #@+node:bob.20110119123023.7413: *3* addLinkItem
-    def addLinkItem(self, from_, to, hierarchyLink = False):
+    def addLinkItem(self, from_, to, hierarchyLink=False):
         if from_ not in self.nodeItem:
             return
         if to not in self.nodeItem:
@@ -931,10 +906,10 @@ class graphcanvasController(object):
         toSize = self.nodeItem[to].size()
 
         li.setLine(
-            from_.u['_bklnk']['x'] + fromSize.width()/2,
-            from_.u['_bklnk']['y'] + fromSize.height()/2+self.nodeItem[from_].iconVPos,
-            to.u['_bklnk']['x'] + toSize.width()/2,
-            to.u['_bklnk']['y'] + toSize.height()/2+self.nodeItem[to].iconVPos,
+            from_.u['_bklnk']['x'] + fromSize.width() / 2,
+            from_.u['_bklnk']['y'] + fromSize.height() / 2 + self.nodeItem[from_].iconVPos,
+            to.u['_bklnk']['x'] + toSize.width() / 2,
+            to.u['_bklnk']['y'] + toSize.height() / 2 + self.nodeItem[to].iconVPos,
             hierarchyLink)
     #@+node:bob.20110127092345.6036: *3* newPos
     def newPos(self, nodeItem, event):
@@ -982,7 +957,7 @@ class graphcanvasController(object):
         ):
             lastNode.bg.setPen(QtGui.QPen(QtConst.NoPen))
 
-        if  (not isinstance(nodeItem, nodeNone) and
+        if (not isinstance(nodeItem, nodeNone) and
              not isinstance(nodeItem, nodeImage)
         ):
             nodeItem.bg.setPen(self.selectPen)
@@ -1013,18 +988,15 @@ class graphcanvasController(object):
         self.c.redraw()
         self.loadGraph(pnt=pnt)
 
-    #@+node:bob.20110119123023.7418: *3* pressLink
+    #@+node:bob.20110119123023.7418: *3* pressLink (graphcanvas.py)
     def pressLink(self, linkItem, event):
         """nodeItem is telling us it was clicked"""
-
         blc = getattr(self.c, 'backlinkController')
-
         if not blc:
             return
-
-        if not (event.modifiers() & QtConst.ControlModifier):
+        # pylint: disable=superfluous-parens
+        if not (event.modifiers() & KeyboardModifier.ControlModifier):
             return
-
         if linkItem in self.link:
             link = self.link[linkItem]
 
@@ -1114,7 +1086,7 @@ class graphcanvasController(object):
                         self.addLinkItem(i, child, hierarchyLink=True)
 
         if adjust:
-            self.ui.canvasView.setSceneRect(self.ui.canvas.sceneRect().adjusted(-50,-50,50,50))
+            self.ui.canvasView.setSceneRect(self.ui.canvas.sceneRect().adjusted(-50, -50, 50, 50))
     #@+node:bob.20110119123023.7422: *3* goto
     def goto(self):
         """make outline select node"""
@@ -1152,8 +1124,8 @@ class graphcanvasController(object):
 
         for i in self.nodeItem:
 
-            i.u['_bklnk']['x'] = midx + (i.u['_bklnk']['x']-midx) * direction
-            i.u['_bklnk']['y'] = midy + (i.u['_bklnk']['y']-midy) * direction
+            i.u['_bklnk']['x'] = midx + (i.u['_bklnk']['x'] - midx) * direction
+            i.u['_bklnk']['y'] = midy + (i.u['_bklnk']['y'] - midy) * direction
             self.nodeItem[i].setPos(i.u['_bklnk']['x'], i.u['_bklnk']['y'])
             self.nodeItem[i].do_update()
 
@@ -1201,7 +1173,7 @@ class graphcanvasController(object):
     #@+others
     #@+node:bob.20110121113659.3412: *4* Events
     #@+node:bob.20110120173002.3405: *5* onSelect2
-    def onSelect2 (self,tag,keywords):
+    def onSelect2(self, tag, keywords):
 
         """Shows the UNL in the status line whenever a node gets selected."""
 
@@ -1213,16 +1185,17 @@ class graphcanvasController(object):
 
         # c.p is not valid while using the settings panel.
         new_p = keywords.get('new_p')
-        if not new_p: return
+        if not new_p:
+            return
 
         if new_p.h.startswith('@graph'):
             self.clear()
-            self.loadGraph('node', create = False)
+            self.loadGraph('node', create=False)
             if '_bklnk' in new_p.v.u:
                 # self.loadLinked('all')
-                self.loadGraph('recur', create = False)
+                self.loadGraph('recur', create=False)
             elif self.lastNodeItem and '_bklnk' in self.lastNodeItem.node.u:
-                x,y = self.lastNodeItem.node.u['_bklnk']['x'], self.lastNodeItem.node.u['_bklnk']['y']
+                x, y = self.lastNodeItem.node.u['_bklnk']['x'], self.lastNodeItem.node.u['_bklnk']['y']
                 self.ui.canvasView.centerOn(x, y)
 
         if c.p.v in self.nodeItem and self.ui.UI.chkTrack.isChecked():
@@ -1237,7 +1210,7 @@ class graphcanvasController(object):
             return
 
         if 'x' in node.u['_bklnk']:
-            x,y = node.u['_bklnk']['x'], node.u['_bklnk']['y']
+            x, y = node.u['_bklnk']['x'], node.u['_bklnk']['y']
 
         self.ui.canvasView.centerOn(x, y)
 
@@ -1277,7 +1250,7 @@ class graphcanvasController(object):
     #@+node:bob.20110202125047.4170: *5* exportGraph
     def exportGraph(self):
 
-        image = QtGui.QImage(2048,1536,QtGui.QImage.Format_ARGB32_Premultiplied)
+        image = QtGui.QImage(2048, 1536, QtGui.QImage.Format_ARGB32_Premultiplied)
         painter = QtGui.QPainter(image)
         self.ui.canvas.render(painter)
         painter.end()
@@ -1339,9 +1312,9 @@ class graphcanvasController(object):
         item = self.nodeItem[node]
         # FIXME: need node.clear_formatting()
         if hasattr(item, 'bg') and hasattr(item.bg, 'setBrush'):
-            item.bg.setBrush(QtGui.QBrush(QtGui.QColor(200,240,200)))
+            item.bg.setBrush(QtGui.QBrush(QtGui.QColor(200, 240, 200)))
         if hasattr(item, 'text'):
-            item.text.setDefaultTextColor(QtGui.QColor(0,0,0))
+            item.text.setDefaultTextColor(QtGui.QColor(0, 0, 0))
         if 'color' in node.u['_bklnk']:
             del node.u['_bklnk']['color']
         if 'tcolor' in node.u['_bklnk']:
