@@ -95,7 +95,7 @@ whitespace (calling .strip()).
 """
 #@-<< docstring >>
 import re
-from typing import Dict, List
+from typing import Dict, Generator, List
 from leo.core import leoGlobals as g
 from leo.core import leoNodes
 from leo.core.leoQt import QtCore, QtWidgets
@@ -205,7 +205,7 @@ class TagController:
                     break
         return nodelist
     #@+node:vitalije.20170811150914.1: *4* tag_c.get_tagged_gnxes
-    def get_tagged_gnxes(self, tag):
+    def get_tagged_gnxes(self, tag: str) -> Generator:
         c = self.c
         tag = tag.replace('*', '.*')
         regex = re.compile(tag)
@@ -262,7 +262,8 @@ if QtWidgets:
             self.search_re = r'(&|\||-|\^)'
             self.custom_searches = []
             g.registerHandler('select2', self.select2_hook)
-            g.registerHandler('create-node', self.command2_hook)  # fix tag jumplist positions after new node insertion
+            # fix tag jumplist positions after new node insertion
+            g.registerHandler('create-node', self.command2_hook)
             g.registerHandler('command2', self.command2_hook)
         #@+node:peckj.20140804114520.15202: *4* tag_w.initUI
         def initUI(self):
