@@ -1,5 +1,7 @@
+# -*- coding: utf-8 -*-
 #@+leo-ver=5-thin
 #@+node:ekr.20140907085654.18699: * @file ../plugins/qt_gui.py
+#@@first
 """This file contains the gui wrapper for Qt: g.app.gui."""
 # pylint: disable=import-error
 #@+<< imports qt_gui.py >>
@@ -950,8 +952,10 @@ class LeoQtGui(leoGui.LeoGui):
         try:
             font = QtGui.QFont(family, i_size, weight_val, italic)
             if sys.platform.startswith('linux'):
-                font.setHintingPreference(font.PreferFullHinting)
-            # g.es(font,font.hintingPreference())
+                try:
+                    font.setHintingPreference(font.PreferFullHinting)
+                except AttributeError:
+                    pass
             return font
         except Exception:
             g.es("exception setting font", g.callers(4))
@@ -960,7 +964,7 @@ class LeoQtGui(leoGui.LeoGui):
                 f"  size: {i_size}\n"
                 f" slant: {slant}\n"
                 f"weight: {weight}")
-            # g.es_exception() # This just confuses people.
+            # g.es_exception() # Confusing for most users.
             return None
     #@+node:ekr.20110605121601.18511: *3* qt_gui.getFullVersion
     def getFullVersion(self, c: Cmdr=None) -> str:
@@ -1134,6 +1138,7 @@ class LeoQtGui(leoGui.LeoGui):
         """
         Create a script button for the script in node p.
         The button's text defaults to p.headString."""
+        # pylint: disable=line-too-long
         k = c.k
         if p and not buttonText:
             buttonText = p.h.strip()
