@@ -75,23 +75,24 @@ This plugin defines the following commands that can be bound to keys:
 
 """
 #@-<< docstring >>
-# Ville M. Vainio <vivainio@gmail.com>.
+# Original by Ville M. Vainio <vivainio@gmail.com>.
 #@+<< imports >>
 #@+node:ville.20090314215508.7: ** << imports >>
-from collections import OrderedDict
 import fnmatch
 import itertools
 import re
+from typing import Dict, List, Tuple, Union, Iterator, Match, Optional
 from leo.core import leoGlobals as g
 from leo.core.leoQt import QtCore, QtConst, QtWidgets
 from leo.core.leoQt import KeyboardModifier
-from leo.core import leoNodes
+from leo.core.leoNodes import Position
 from leo.plugins import threadutil
 from leo.plugins import qt_quicksearch_sub as qt_quicksearch
 #
 # Fail fast, right after all imports.
 g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
 #@-<< imports >>
+RegexFlag = Union[int, re.RegexFlag]  # re.RegexFlag does not define 0
 #@+others
 #@+node:ekr.20190210123045.1: ** top level
 #@+node:ville.20121223213319.3670: *3* dumpfocus (quicksearch.py)
@@ -186,7 +187,7 @@ def install_qt_quicksearch_tab(c):
         tab_widget = wdg.parent().parent()
         tab_widget.currentChanged.connect(activate_input)
 #@+node:ekr.20111014074810.15659: *3* matchLines
-def matchlines(b, miter):
+def matchlines(b: str, miter: Iterator[Match[str]]):
 
     res = []
     for m in miter:
@@ -232,31 +233,6 @@ def show_unittest_failures(event):
             it.setToolTip(stack)
 
     c.k.simulateCommand('focus-to-nav')
-#@+node:jlunz.20151027094647.1: ** class OrderedDefaultDict (OrderedDict)
-class OrderedDefaultDict(OrderedDict):
-    """
-    Credit:  http://stackoverflow.com/questions/4126348/
-    how-do-i-rewrite-this-function-to-implement-ordereddict/4127426#4127426
-    """
-    def __init__(self, *args, **kwargs):
-        if not args:
-            self.default_factory = None
-        else:
-            if not (args[0] is None or callable(args[0])):
-                raise TypeError('first argument must be callable or None')
-            self.default_factory = args[0]
-            args = args[1:]
-        super().__init__(*args, **kwargs)
-
-    def __missing__(self, key):
-        if self.default_factory is None:
-            raise KeyError(key)
-        self[key] = default = self.default_factory()
-        return default
-
-    def __reduce__(self):  # optional, for pickle support
-        args = (self.default_factory,) if self.default_factory else ()
-        return self.__class__, args, None, None, self.items()
 #@+node:ekr.20111015194452.15716: ** class QuickSearchEventFilter (QObject)
 class QuickSearchEventFilter(QtCore.QObject):  # type:ignore
 
@@ -360,7 +336,7 @@ class LeoQuickSearchWidget(QtWidgets.QWidget):  # type:ignore
     def selectAndDismiss(self):
         self.hide()
     #@-others
-#@+node:ville.20090314215508.12: ** class QuickSearchController
+#@+node:ville.20090314215508.12: ** class QuickSearchController (quicksearch.py)
 class QuickSearchController:
 
     #@+others
@@ -378,7 +354,10 @@ class QuickSearchController:
         self.frozen = False
         self._search_patterns = []
 
-        def searcher(inp):
+        def searcher(inp:str) -> Tuple[
+            List[Tuple[Position, Optional[Iterator[Match[str]]]]],
+            List[Tuple[Position, Optional[Iterator[Match[str]]]]]
+        ]:
             #print("searcher", inp)
             if self.frozen:
                 return None
@@ -386,14 +365,17 @@ class QuickSearchController:
             res = self.bgSearch(exp)
             return res
 
-        def dumper():
+        def dumper() -> None:
             # always run on ui thread
             if self.frozen:
                 return
             out = self.worker.output
             self.throttler.add(out)
 
-        def throttledDump(lst):
+        def throttledDump(lst: List[Tuple[
+            List[Tuple[Position, Optional[Iterator[Match[str]]]]],
+            List[Tuple[Position, Optional[Iterator[Match[str]]]]]]
+        ]) -> None:
             """ dumps the last output """
             # we do get called with empty list on occasion
             if not lst:
@@ -425,24 +407,27 @@ class QuickSearchController:
         self.its[id(it)] = val
         return len(self.its) > 300
     #@+node:ekr.20111015194452.15689: *3* addBodyMatches
-    def addBodyMatches(self, poslist):
+    def addBodyMatches(self, positions:  List[Tuple[Position, Optional[Iterator[Match[str]]]]]) -> int:
         lineMatchHits = 0
-        for p in poslist:
-            it = QtWidgets.QListWidgetItem(p.h, self.lw)
+        for p in positions:
+            it = QtWidgets.QListWidgetItem(p[0].h, self.lw)
             f = it.font()
             f.setBold(True)
             it.setFont(f)
-            if self.addItem(it, (p, None)):
+            if self.addItem(it, (p[0], None)):
                 return lineMatchHits
-            ms = matchlines(p.b, p.matchiter)
+
+            ms = matchlines(p[0].b, p[1])
             for ml, pos in ms:
                 lineMatchHits += 1
                 it = QtWidgets.QListWidgetItem("    " + ml, self.lw)
-                if self.addItem(it, (p, pos)):
+                if self.addItem(it, (p[0], pos)):
                     return lineMatchHits
         return lineMatchHits
     #@+node:jlunz.20151027092130.1: *3* addParentMatches
-    def addParentMatches(self, parent_list):
+    def addParentMatches(self,
+        parent_list: Dict[str,  List[Tuple[Position, Optional[Iterator[Match[str]]]]]]
+    ) -> int:
         lineMatchHits = 0
         for parent_key, parent_value in parent_list.items():
             if isinstance(parent_key, str):
@@ -450,25 +435,25 @@ class QuickSearchController:
                 h = v.h if v else parent_key
                 it = QtWidgets.QListWidgetItem(h, self.lw)
             else:
-                it = QtWidgets.QListWidgetItem(parent_key.h, self.lw)
+                it = QtWidgets.QListWidgetItem(parent_key[0].h, self.lw)
             f = it.font()
             f.setItalic(True)
             it.setFont(f)
             if self.addItem(it, (parent_key, None)):
                 return lineMatchHits
             for p in parent_value:
-                it = QtWidgets.QListWidgetItem("    " + p.h, self.lw)
+                it = QtWidgets.QListWidgetItem("    " + p[0].h, self.lw)
                 f = it.font()
                 f.setBold(True)
                 it.setFont(f)
-                if self.addItem(it, (p, None)):
+                if self.addItem(it, (p[0], None)):
                     return lineMatchHits
-                if hasattr(p, "matchiter"):  #p might be not have body matches
-                    ms = matchlines(p.b, p.matchiter)
+                if p[1] is not None:  #p might not have body matches
+                    ms = matchlines(p[0].b, p[1])
                     for ml, pos in ms:
                         lineMatchHits += 1
                         it = QtWidgets.QListWidgetItem("    " + "    " + ml, self.lw)
-                        if self.addItem(it, (p, pos)):
+                        if self.addItem(it, (p[0], pos)):
                             return lineMatchHits
         return lineMatchHits
 
@@ -479,14 +464,14 @@ class QuickSearchController:
         self.its[id(it)] = f
         return it
     #@+node:ekr.20111015194452.15688: *3* addHeadlineMatches
-    def addHeadlineMatches(self, poslist):
+    def addHeadlineMatches(self, poslist: List[Tuple[Position, Optional[Iterator[Match[str]]]]]):
 
         for p in poslist:
-            it = QtWidgets.QListWidgetItem(p.h, self.lw)
+            it = QtWidgets.QListWidgetItem(p[0].h, self.lw)
             f = it.font()
             f.setBold(True)
             it.setFont(f)
-            if self.addItem(it, (p, None)):
+            if self.addItem(it, (p[0], None)):
                 return
     #@+node:ekr.20111015194452.15691: *3* clear
     def clear(self):
@@ -495,9 +480,12 @@ class QuickSearchController:
         self.lw.clear()
 
     #@+node:ekr.20111015194452.15693: *3* doNodeHistory
-    def doNodeHistory(self):
+    def doNodeHistory(self) -> None:
 
-        nh = leoNodes.PosList(po[0] for po in self.c.nodeHistory.beadList)
+        c = self.c
+        nh: List[Tuple[Position, Optional[Iterator[Match[str]]]]] = [
+            (z[0].copy(), None) for z in c.nodeHistory.beadList
+        ]
         nh.reverse()
         self.clear()
         self.addHeadlineMatches(nh)
@@ -522,15 +510,19 @@ class QuickSearchController:
     def doTimeline(self):
 
         c = self.c
-        timeline = [p.copy() for p in c.all_unique_positions()]
-        timeline.sort(key=lambda x: x.gnx, reverse=True)
+        timeline: List[Tuple[Position, Optional[Iterator[Match[str]]]]] = [
+            (p.copy(), None) for p in c.all_unique_positions()
+        ]
+        timeline.sort(key=lambda x: x[0].gnx, reverse=True)
         self.clear()
         self.addHeadlineMatches(timeline)
     #@+node:tbrown.20131204085704.57542: *3* doChanged
     def doChanged(self):
 
         c = self.c
-        changed = [p.copy() for p in c.all_unique_positions() if p.isDirty()]
+        changed: List[Tuple[Position, Optional[Iterator[Match[str]]]]] = [
+            (p.copy(), None) for p in c.all_unique_positions() if p.isDirty()
+        ]
         self.clear()
         self.addHeadlineMatches(changed)
     #@+node:ekr.20111015194452.15692: *3* doSearch
@@ -602,17 +594,23 @@ class QuickSearchController:
         if not hitBase:
             hm = self.find_h(hpat, hNodes, flags)
             bm = self.find_b(bpat, bNodes, flags)
-            bm_keys = [match.key() for match in bm]
+            bm_keys = [match[0].key() for match in bm]
             numOfHm = len(hm)  #do this before trim to get accurate count
-            hm = [match for match in hm if match.key() not in bm_keys]
+            hm = [match for match in hm if match[0].key() not in bm_keys]
             if self.widgetUI.showParents.isChecked():
-                parents = OrderedDefaultDict(list)
+                ### parents = OrderedDefaultDict(list)
+                parents: Dict[str, List[Tuple[Position, Optional[Iterator[Match[str]]]]]] = {}
                 for nodeList in [hm, bm]:
                     for node in nodeList:
-                        if node.level() == 0:
-                            parents["Root"].append(node)
-                        else:
-                            parents[node.parent().gnx].append(node)
+                        ###
+                            # if node.level() == 0:
+                                # parents["Root"].append(node)
+                            # else:
+                                # parents[node.parent().gnx].append(node)
+                        key = 'Root' if node[0].level() == 0 else node[0].parent().gnx
+                        aList: List[Tuple[Position, Optional[Iterator[Match[str]]]]] = parents.get(key, [])
+                        aList.append(node)
+                        parents[key] = aList
                 lineMatchHits = self.addParentMatches(parents)
             else:
                 self.addHeadlineMatches(hm)
@@ -626,8 +624,10 @@ class QuickSearchController:
                 self.lw.insertItem(0, "External file directive not found " +
                                       "during search")
     #@+node:ville.20121118193144.3620: *3* bgSearch
-    def bgSearch(self, pat):
-
+    def bgSearch(self, pat:str) -> Tuple[
+        List[Tuple[Position, Optional[Iterator[Match[str]]]]],
+        List[Tuple[Position, Optional[Iterator[Match[str]]]]]
+    ]:
         #self.clear()
 
         if self.frozen:
@@ -654,34 +654,36 @@ class QuickSearchController:
         return hm, []
         # self.lw.insertItem(0, "%d hits"%self.lw.count())
     #@+node:jlunz.20150826091415.1: *3* find_h
-    def find_h(self, regex, nodes, flags=re.IGNORECASE):
-        """ Return list (a PosList) of all nodes where zero or more characters at
-        the beginning of the headline match regex
+    def find_h(self,
+        regex: str,
+        positions: List[Position],
+        flags: RegexFlag=re.IGNORECASE,
+    ) -> List[Tuple[Position, Optional[Iterator[Match[str]]]]]:
+        """
+        Return the list of all tuple (Position, matchiter/None) whose headline matches the given pattern.
         """
-        res = leoNodes.PosList()
         try:
             pat = re.compile(regex, flags)
         except Exception:
-            return res
-        for p in nodes:
-            m = re.match(pat, p.h)
-            if m:
-                # #2012: Don't inject pc.mo.
-                pc = p.copy()
-                res.append(pc)
-        return res
+            return []
+        return [(p.copy(), None) for p in positions if re.match(pat, p.h)]
     #@+node:jlunz.20150826091424.1: *3* find_b
-    def find_b(self, regex, nodes, flags=re.IGNORECASE | re.MULTILINE):
-        """ Return list (a PosList) of all nodes whose body matches regex
-        one or more times.
-
+    def find_b(self,
+        regex: str,
+        positions: List[Position],
+        flags: RegexFlag=re.IGNORECASE | re.MULTILINE,
+    ) -> List[Tuple[Position, Optional[Iterator[Match[str]]]]]:
+        """
+        Return list of all tuple (Position, matchiter/None) whose body matches regex one or more times.
         """
-        res = leoNodes.PosList()
         try:
             pat = re.compile(regex, flags)
         except Exception:
-            return res
-        for p in nodes:
+            return []
+
+        aList: List[Tuple[Position, Optional[Iterator[Match[str]]]]] = []
+
+        for p in positions:
             m = re.finditer(pat, p.b)
             t1, t2 = itertools.tee(m, 2)
             try:
@@ -689,19 +691,18 @@ class QuickSearchController:
             except StopIteration:
                 continue
             pc = p.copy()
-            pc.matchiter = t2
-            res.append(pc)
-        return res
+            aList.append((pc, t2))
+
+        return aList
+
     #@+node:ekr.20111015194452.15687: *3* doShowMarked
     def doShowMarked(self):
 
         self.clear()
         c = self.c
-        pl = leoNodes.PosList()
-        for p in c.all_positions():
-            if p.isMarked():
-                pl.append(p.copy())
-        self.addHeadlineMatches(pl)
+        self.addHeadlineMatches([
+            (p.copy(), None) for p in c.all_positions()if p.isMarked()
+        ])
     #@+node:ekr.20111015194452.15700: *3* Event handlers
     #@+node:ekr.20111015194452.15686: *4* onSelectItem (quicksearch.py)
     def onSelectItem(self, it, it_prev=None):
